package sears.file;
/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

//some suggestions about this class: floriaen@gmail.com

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;

import sears.file.exception.io.FileConversionException;

/**
 * This class must be implemented each time a <tt>java.io.File</tt> object will have to be converted to
 * a <tt>sears.file.SubtitleFile</tt> object
 * <br>It wrap a <tt>BufferedReader</tt> and gives method to access to it with respect of the Sears constraints
 */
public abstract class FileConversion {

	protected File file;
	private BufferedReader reader;
	protected int lineCount;

	/**
	 * Constructs a new instance 
	 * @param file		the file to parse
	 * @param charset	the charset used to read the file
	 * @throws NullPointerException if <tt>file</tt> or/and <tt>charset</tt> are null
	 * @throws FileConversionException
	 */
	public FileConversion(File file, String charset) throws FileConversionException {
		if( file == null ) {
			throw new NullPointerException("cannot parse a null file");
		}		
		if( charset == null ) {
			throw new NullPointerException("charset could not be null");
		}
		this.file = file;
		initializeReader(charset);
	}
	
	// **************
	// CORE METHODS *
	// **************
	/**
	 * <br>Principal method:
	 * <br>parses <tt>file</tt> and fill the array list given in parameters with founded subtitles informations
	 * <br>If there's no subtitles found, the array list stays as it was before
	 * @param subtitleList					the array of subtitles to fill
	 * @throws FileConversionException		if an error occurs during the conversion
	 */
	public void parse(ArrayList<Subtitle> subtitleList) throws FileConversionException {
		// while there's line to read:
		String line = "";
		while( (line = readLine()) != null ) {
			Subtitle subtitle = getSubtitle(line);
			if( subtitle != null ) {
				subtitleList.add(subtitle);
			} //else line is empty line, loop
		}
		if( subtitleList.isEmpty() ) {
			//throw new EmptySubtitleFileException(file);
			throw FileConversionException.getMalformedSubtitleFileException(
					FileConversionException.EMPTY_SUBTITLE_FILE, file);
		}
	}
	
	/**
	 * Constructs and returns the <tt>Subtitle</tt> object which represents subtitles information
	 * begin at line given in parameters
	 * <br>This method is used by the non <tt>abstract</tt> {@link #parse(ArrayList)} method
	 * @param line						the first line...
	 * @return							the founded subtitle or null if there's no subtitle found
	 * @throws FileConversionException	if an error occurs during the conversion
	 */
	protected abstract Subtitle getSubtitle(String line) throws FileConversionException;

	
	// ***********************
	// READER ACCESS METHODS *
	// ***********************
	/**
	 * Initializes the reader with the <tt>charset</tt> given in parameters
	 * @param charset					if null there's no guarantee to the coherence of this method
	 * @throws FileConversionException	if an error occurs during the initialisation
	 */
	private void initializeReader(String charset) throws FileConversionException {
		try {
			lineCount = 0;
			reader = new BufferedReader(new InputStreamReader(new FileInputStream(file), charset));
		} catch ( UnsupportedEncodingException e ) {
			// fsa.unsupportedEncoding(file, charset);
			e.printStackTrace();
		} catch ( FileNotFoundException e) {
			throw FileConversionException.getAccessException(
					FileConversionException.READ_ACCESS, file);
		}
	}
	
	/**
	 * Reads and returns the next line in the buffer and increment the line count
	 * @return							the next line or null if there's no more line to read
	 * @throws FileConversionException 	if a basic <tt>IOException</tt> occurs when 
	 * 									attempts to read in the buffer
	 */
	protected String readLine() throws FileConversionException {
		String line = null;
		lineCount++;
		try {
			line = reader.readLine();
		} catch (IOException e) {
			throw FileConversionException.getAccessException(
					FileConversionException.READ_ACCESS, file);
		}
		return line;
	}
	
	/**
	 * Place the 'read head' on the next non empty line and returns it
	 * @return the next non empty line or null if there's no more line to read
	 */
	protected String getTheNextNonEmptyLine() throws FileConversionException {
		String str = null;
		try {
			str = readLine();
			while( str != null && str.equals("") ){
				str = readLine();
			}
		} catch (IOException e) {
			throw FileConversionException.getAccessException(
					FileConversionException.READ_ACCESS, file);
		}
		return str;
	}
	
	/**
	 * Closes the reader and reset line count
	 * @throws FileConversionException 	if a basic <tt>IOException</tt> occurs when 
	 * 									attempts to close the the buffer reader
	 */
	protected void closeReader() throws FileConversionException {
		if( reader != null ) {
			try {
				reader.close();
				lineCount = 0;
			} catch (IOException e) {
				// try to inform that error occurs when closed the file
				//throw new ReadAccessException(file);
				throw FileConversionException.getAccessException(
						FileConversionException.READ_ACCESS, file);
			}
		}
	}

	// ***************
	// UTILS METHODS *
	// ***************
	/**
	 * Tests the validity of <tt>str</tt>
	 * return 	true if <tt>str</tt> is not null and if it is not an empty <tt>String</tt>, false if not
	 * @throws IOException 
	 */
	protected void ensureStringIsValid(String str) throws FileConversionException {
		if ( str == null || str.equals("")) {
			//throw new MalformedSubtitleFileException(file, lineCount, "");
			throw FileConversionException.getMalformedSubtitleFileException(
					FileConversionException.MALFORMED_SUBTITLE_FILE, 
					file, lineCount, "");
		}
	}

	/**
	 * If <tt>str</tt> length is more than 100 characters, <tt>str</tt> is cut and returned
	 * @param str	the string to cut
	 * @return		<tt>str</str> or a sub string of this array
	 */
	protected static String subString(String str) {
		if( str.length() > 100 ) {
			str = str.substring(0, 100);
		}
		return str;
	}
}