package sears.file;

import java.io.File;
import java.util.ArrayList;

import sears.file.exception.io.FileConversionException;
import sears.gui.MainWindow;
import sears.tools.Utils;

/**
 * Provides access method to file like <i>open</i> and <i>save</i>
 * <br>An instance of this class captures error coming from the conversion of file to subtitle file
 */
public class FileSystemAccess {

	private static FileSystemAccess instance;
	private MainWindow mainWindow;

	/** (<tt>int</tt>) maximum authorized size for a subtitle file, 2 Mo */
	public final static int MAX_SUBTITLE_FILE_SIZE = 2000000;

	/**
	 * Gives the same instance of this class
	 * <br>Only used by <tt>MainWindow</tt> class
	 * @return	an object instance of <tt>FileSystemAccess</tt> class.
	 */
	public static FileSystemAccess getInstance() {
		if( instance == null ) {
			instance = new FileSystemAccess(MainWindow.instance);
		}
		return instance;
	}

	/**
	 * Default constructor
	 * @param mainWindow	the extended controller
	 */
	protected FileSystemAccess(MainWindow mainWindow) {
		if( mainWindow == null ) {
			throw new NullPointerException("main window cannot be null");
		}
		this.mainWindow = mainWindow;
	}

	/**
	 * Tests the length of the <tt>java.io.File</tt> given in parameters
	 * @param file						the file to test
	 * @throws FileConversionException	if length or access to the file cause a problem	
	 * @throws NullPointerException 	if <tt>file</tt> is null
	 */
	private void ensureFileLengthCoherence(File file) throws FileConversionException {
		try {
			if( file.length() >= MAX_SUBTITLE_FILE_SIZE ) {
				throw FileConversionException.getAccessException(
						FileConversionException.UNSUPPORTED_FILE_SIZE, file);
			}
		} catch( SecurityException e) {
			throw FileConversionException.getAccessException(
					FileConversionException.READ_ACCESS, file);
		}
	}
	
	/**
	 * Provides a way to get a <tt>SubtitleFile</tt> instance from a file
	 * @param file		the file to convert
	 * @param charset	the charset used for the conversion
	 * @return			the <tt>SubtitleFile</tt> object created or null if an error occurs
	 */
	public SubtitleFile openFile(File file, String charset) {
		return openFile(file, null, charset);
	}
	
	private void fillArray(ArrayList<Subtitle> listToFill, ArrayList<Subtitle> savedList) {
		listToFill.clear();
		listToFill.addAll(savedList);
	}

	/**
	 * Provides a way to get <tt>SubtitleFile</tt> instance from a file
	 * <br>If an error occurs an error dialog is called
	 * @param file			the file to convert
	 * @param subtitleList	the subtitle list to fill
	 * @param charset		the charset used for the conversion
	 * @return				the <tt>SubtitleFile</tt> object created or null if an error occurs
	 */
	public SubtitleFile openFile(File file, ArrayList<Subtitle> subtitleList, String charset) {
		SubtitleFile subtitleFile = null;
		// save the subtitle list in case of error:
		ArrayList<Subtitle> savedSubtitleList = new ArrayList<Subtitle>();
		if( subtitleList == null ) {
			// subtitleList becomes an empty array
			subtitleList = savedSubtitleList;
		} else {			
			savedSubtitleList.addAll(subtitleList);
			subtitleList.clear();
		}
		// Try to create the new subtitleFile:
		try {	
			// ensure that the file length is not bigger than the one supported by Sears
			ensureFileLengthCoherence(file);
			// try to create a new Subtitle file
			subtitleFile = SubtitleFile.getInstance(file, subtitleList, charset);
		} catch ( FileConversionException e ) {
			//
			// file conversion error occurs:
			// - malformed subtitle file
			// - read access error
			
			// return saved list
			fillArray(subtitleList, savedSubtitleList);
			charset = mainWindow.showOpenErrorDialog(e.getMessage(), e.getDetail(), e.couldBeABadEncoding());
			if( charset != null ) {
				mainWindow.openFile(file, charset);
			} // else, error dialog has be closed by user, do not try to reopen file
		} catch ( NullPointerException e) {
			//
			// DEVELOPER:
			// one of the parameters is null !!
			
			// return saved list
			fillArray(subtitleList, savedSubtitleList);
			e.printStackTrace();
			mainWindow.showOpenErrorDialog("unknow error", e.toString(), false);
			
		} catch ( Exception e ) {
			//
			// DEVELOPER:
			// if an unknown error occurs, a generic dialog is show...
			
			// return saved list
			fillArray(subtitleList, savedSubtitleList);
			e.printStackTrace();
			mainWindow.showOpenErrorDialog("unknow error",e.toString(), false);			
		}
		return subtitleFile;
	}

	/**
	 * Writes the subtitle to a file and invokes an error dialog if an error occurs during the action
	 * @param file			the file in which the subtitle must be write
	 * @param subtitleFile	the subtitle to write
	 * @param charset		the charset used to write the file, if null the subtitle's charset will be used
	 * @return				true if action succeed, false if not
	 */
	public boolean saveFile(File file, SubtitleFile subtitleFile, String charset) {
		boolean isFileSaved = false;
		if (subtitleFile != null) {
			if( charset != null ) {
				subtitleFile.setCharset(charset);
			} // else the file will be saved with this defined charset
			try {
				//we have to check wether we are saving into the same format
				if(subtitleFile.getFile().getName().endsWith(Utils.getExtension(file))){
					//If same extension, save the file
					subtitleFile.writeToFile(file);
				}else{
					//If not the same extension, create the new subtitle file., using the empty constructor of getInstance facility (true)
					SubtitleFile newFile = SubtitleFile.getInstance(file, null, null, true);
					//File has not been set by empty constructor, set it
					newFile.setFile(file);
					//copy subtitle list
					newFile.setSubtitleList(subtitleFile.getSubtitleList());
					//And save the new file
					newFile.writeToFile(file);
				}
				isFileSaved = true;
			} catch (FileConversionException e) {
				mainWindow.showSaveErrorDialog(e.getMessage(), e.getDetail());
			} catch (Exception e) {
				mainWindow.showSaveErrorDialog(e.getMessage(),e.getLocalizedMessage());
			}
		}		
		return isFileSaved;
	}
}