/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com

package sears.file;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.StringTokenizer;

import sears.file.exception.io.FileConversionException;
import sears.tools.Trace;
import sears.tools.Utils;

/**
 * Class SrtFile.
 * <br><b>Summary:</b><br>
 * This class represent a srt subtitle file.
 * Specialize the SubtitleFile for srt type subtitles.
 */
public class SrtFile extends SubtitleFile {

	protected static final String TIME_SEPARATOR = " --> ";

	protected static final String TOKEN_TIME_DELIMITER = "-> ";

	protected static final String TIME_PATTERN = "^((\\d{2}\\:d{2}){2}\\d{2}\\,\\d{3})";

	//protected static final String TIME_LINE_PATTERN = "";

	/**
	 * Constructor SrtFile.
	 * <br><b>Summary:</b><br>
	 * Constructor of the class.
	 * Beware not to use this file directly, because it does contains no ST.
	 * You will have to fill the list of ST, and save the File first.
	 */

	public SrtFile(){
		super();
	}

	/**
	 * Constructor SrtFile.
	 * <br><b>Summary:</b><br>
	 * Constructor of the class.
	 * @param file            The <b>(File)</b> to open.
	 * @param subtitleList        The <b>(ArrayList)</b> List of subtitles.
	 * @throws IOException 
	 * @throws MalformedSubtitleFileException 
	 */
	public SrtFile(File file, ArrayList<Subtitle> subtitleList) throws FileConversionException {
		super(file, subtitleList);
	}

	/**
	 * Constructor SrtFile.
	 * <br><b>Summary:</b><br>
	 * Constructor of the class.
	 * @param file                The <b>(String)</b> path to file to open.
	 * @param subtitleList        The <b>(ArrayList)</b> List of subtitles.
	 * @throws IOException 
	 * @throws MalformedSubtitleFileException 
	 */
	public SrtFile(String file, ArrayList<Subtitle> subtitleList) throws FileConversionException {
		super(file, subtitleList);
	}

	public SrtFile(File file, ArrayList<Subtitle> subtitleList, String charset) throws FileConversionException {
		super(file, subtitleList, charset);
	}

	/* (non-Javadoc)
	 * @see sears.file.SubtitleFile#parse()
	 */
	protected void parse() throws FileConversionException {		
		
		FileConversion pm = null;
		String charset = DEFAULT_CHARSET;
		if( !charset.equals(getCharset()) ) {
			pm = new FileToSrtFile(file, getCharset());
			// try to fill the list of subtitles:
			pm.parse(subtitleList);
		} else {	
			int count = -1;
			while( count < BASIC_CHARSETS.length ) {
				try {
					pm = new FileToSrtFile(file, charset);
					// try to fill the list of subtitles:
					pm.parse(subtitleList);
					// parse succeed so, stop the loop:
					count = BASIC_CHARSETS.length;
					setCharset(charset);
				} catch (FileConversionException e) {
					// NOTE FOR DEVELOPER:
					//
					// the catching exception must be tested
					// if the nature of the nature is a basic IO exception
					// it is not necessary to parse again with a different charset
					// the error will occur again					
					count++;
					if( count >= BASIC_CHARSETS.length ) {
						// stop trying to decode:
						throw e;
					} else {
						// change the charset
						charset = BASIC_CHARSETS[count];
					}
				}
			}
		}
	}

	/* (non-Javadoc)
	 * @see sears.file.SubtitleFile#writeToFile(java.io.File)
	 */
	public void writeToFile(File fileToWrite) throws FileConversionException {
		{
			//First is to know which end of line to use.
			//by default use the linux one.
			String lineSeparator = getLineSeparator();
			try {
				// keep the initial encoding:
				BufferedWriter out = new BufferedWriter(
						new OutputStreamWriter(
								new FileOutputStream(fileToWrite), super.getCharset()));
				Iterator<Subtitle> subtitles = subtitleList.iterator();
				while (subtitles.hasNext()) {
					Subtitle subtitle = subtitles.next();
					out.write(subtitle.getNumber() + lineSeparator);
					out.write(timeToString(subtitle.getStartDate()) + TIME_SEPARATOR + timeToString(subtitle.getEndDate()) + lineSeparator);
					out.write(subtitle.getSubtitle().replaceAll(Utils.LINE_SEPARATOR, lineSeparator));
					//Check wether subtitle has an end of line.
					if( !subtitle.getSubtitle().endsWith(Utils.LINE_SEPARATOR) ){
						//If it doesn't have one, put one.
						out.write(lineSeparator);
					}
					out.write(lineSeparator);
				}
				out.close();
				//indicate file is good.
				fileChanged = false;
			} catch( IOException e ) {
				// FileNotFoundException
				// IOException
				throw FileConversionException.getAccessException(
						FileConversionException.WRITE_ACCESS, file);
			} catch( SecurityException e) {
				throw FileConversionException.getAccessException(
						FileConversionException.WRITE_ACCESS, file);
			} catch( NumberFormatException e ) {
				// corrupt time in arraylist !!
				e.printStackTrace();
			}
		}
	}

	/* (non-Javadoc)
	 * @see sears.file.SubtitleFile#writeToFile(java.io.File)
	 */
	public void writeToTemporaryFile() {
		try {
			// Create the temporary subtitle file
			if (temporaryFile == null) {
				temporaryFile = java.io.File.createTempFile("SRTSubtitle", null);
				temporaryFile.deleteOnExit();
			}
			// Store to the temporary file
			boolean oldFileChangedStatus = fileChanged;
			writeToFile(temporaryFile);
			// Restore the old file changed value
			fileChanged = oldFileChangedStatus;
		} catch (IOException e) {
			Trace.trace("Error while writing temporary SRT file !",
					Trace.ERROR_PRIORITY);
			Trace.trace(e.getMessage(), Trace.ERROR_PRIORITY);
		}
	}

	/* (non-Javadoc)
	 * @see sears.file.SubtitleFile#getNewInstance()
	 */
	protected SubtitleFile getNewInstance() {
		return new SrtFile();
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.SubtitleFile#extension()
	 */
	public String extension() {
		return "srt";
	}
}

class FileToSrtFile extends FileConversion {

	public static final int START_DATE = 0;
	public static final int END_DATE = 1;

	public FileToSrtFile(File file, String charset) throws FileConversionException {
		super(file, charset);
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.ParseModule#getSubtitle(java.lang.String)
	 */
	protected Subtitle getSubtitle(String line) throws FileConversionException {
		Subtitle newSubtitle = null;
		boolean endOfFile = false;
		if( line.trim().length() == 0 ) {
			// line is empty, try to go to the next non empty line
			line = getTheNextNonEmptyLine();
			if( line == null ) {
				// END OF FILE
				// WE STOP PARSING FILE
				endOfFile = true;	
			}			
		}
		if( !endOfFile ) {			
			// *************
			// GETS NUMBER *
			// *************			
			// Read the first line, it must contain a number
			// line is non null and it is a non empty line
			// if the line is not a number, an exception is throw by the method
			int numberFound = getSubtitleNumberFromString(line);


			// ****************
			// GETS TIME LINE *
			// ****************
			// Read the next non empty line, it must contain the start and end date
			line = getTheNextNonEmptyLine();
			if( line == null ) {
				// there's no more line where a time line must be:
				throw FileConversionException.getMalformedSubtitleFileException(
						FileConversionException.NO_SUBTITLE_TIME, file, lineCount, line);
			}		
			StringTokenizer stk = getTime(line, SrtFile.TOKEN_TIME_DELIMITER);

			// *****************
			// GETS START DATE *
			// *****************
			// if an error occurs when convert string information to date
			// a MalformedSubtitleFileException is throw
			int startDate = getDateFromString(stk.nextToken(), START_DATE);

			// ***************
			// GETS END DATE *
			// ***************
			// if an error occurs when convert string information to date
			// a MalformedSubtitleFileException is throw
			int endDate = getDateFromString(stk.nextToken(), END_DATE);

			// ********************
			// GETS SUBTITLE TEXT *
			// ********************
			// First is to read till we found a subtitle, to avoid error in files with an empty line
			// after the time definition.
			line = getTheNextNonEmptyLine();
			// AUTO CORRECTION, subtitle text isn't found, it become an empty string
			if( line == null ) {
				line = "";
			}
			// here we have found a subtitle, or reached end of file.
			String subtitle = getSubtitleTextFromString(line);
			// creates the new subtitle:
			newSubtitle = new Subtitle(numberFound, startDate, endDate, subtitle);
		}

		return newSubtitle;
	}

	protected StringTokenizer getTime(String str, String timeDelimiter) throws FileConversionException {
		StringTokenizer stk = null;
		ensureStringIsValid(str);
		stk = new StringTokenizer(str, timeDelimiter);
		if( stk.countTokens() != 2 ) {
			// there's more or less information than start and end date (if there is :) )
			throw FileConversionException.getMalformedSubtitleFileException(
					FileConversionException.MALFORMED_TIME_LINE, file, lineCount, subString(str));
		}
		return stk;
	}

	private int getSubtitleNumberFromString(String str) throws FileConversionException {
		int subtitleNumber = 0;
		try {
			subtitleNumber =  Integer.parseInt(str);
			Trace.trace("Number: " + subtitleNumber, Trace.ALGO_PRIORITY);
		} catch ( NumberFormatException e ) {
			// str do not correspond to a number
			// "sub string str in case of the string length is too bigger (process safe)"
			// throw new NoSubtitleNumberException(file, lineCount, subString(str));
			throw FileConversionException.getMalformedSubtitleFileException(
					FileConversionException.MALFORMED_SUBTITLE_NUMBER, 
					file, lineCount, subString(str));
		}
		return subtitleNumber;
	}

	private int getDateFromString(String str, int dateType) throws FileConversionException {
		int date = 0;
		// DEVELOPER:
		//
		// catch exception means that 'this' creates an object.
		// This object creates another object (ParseSubtitleFileException instance).
		// Maybe (sure) is too much, some test in #stringToTime method could be create 
		// directly the FileConversionException object and so, a more efficient error report
		// could be done.
		//
		if( dateType == START_DATE ) {
			try {
				date = SubtitleFile.stringToTime(str);
			} catch ( NumberFormatException e) {
				//throw new MalformedStartTimeException(file, lineCount,  subString(str));
				throw FileConversionException.getMalformedSubtitleFileException(
						FileConversionException.MALFORMED_START_TIME, 
						file, lineCount, subString(str));
			}
		} else if( dateType == END_DATE ) {
			try {
				date = SubtitleFile.stringToTime(str);
			} catch ( NumberFormatException e) {
				//throw new MalformedEndTimeException(file, lineCount,  subString(str));
				throw FileConversionException.getMalformedSubtitleFileException(
						FileConversionException.MALFORMED_END_TIME, 
						file, lineCount, subString(str));
			}
		} else {
			throw new IllegalArgumentException(dateType 
					+ " is not a valid type of date, please see the class constants");
		}
		return date;
	}

	private String getSubtitleTextFromString(String str) throws FileConversionException {
		String subtitle = "";
		// add the first subtitle line
		subtitle += str + Utils.LINE_SEPARATOR;
		// and read the rest of the subtitle.
		for( str = readLine(); ( str != null && !str.equals("") ); str = readLine() ) {
			subtitle += str + Utils.LINE_SEPARATOR;
		}
		return subtitle;
	}	


}