/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

package sears.file;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.StringTokenizer;

import sears.Version;
import sears.file.exception.io.FileConversionException;
import sears.tools.SearsProperties;
import sears.tools.Trace;
import sears.tools.Utils;

/**
 * This class represents a ssa subtitle file.
 * Specialize the SubtitleFile for ssa type subtitles.
 * It represents too an ass subtitle file too.
 */
public class SsaFile extends SubtitleFile {

	public static final String DIALOGUE_KEY = "Dialogue:";

	public static final String FORMAT_KEY = "Format:";
	
	private static final String START_KEY = "Start";
	private static final String END_KEY = "End";
	private static final String TEXT_KEY = "Text";
	/**Properties to be used to generate a SSA file from Sears*/
	private static final String SSA_BEGIN_PART_PROPERTY ="ssa.format.begin";
	private static final String SSA_BEGIN_PART_DEFAULT ="[Script Info]"+SubtitleFile.getLineSeparator()
														+";SSA script originally generated by Sears "+Version.VERSION+SubtitleFile.getLineSeparator()
														+";If script can't be read by you player, please send bug (and script correction) to booba.skaya@gmail.com "+Version.VERSION+SubtitleFile.getLineSeparator()
														+";Thanks o/"+SubtitleFile.getLineSeparator()
														+"ScriptType: v4.00"+SubtitleFile.getLineSeparator()
														+"Collisions: Normal"+SubtitleFile.getLineSeparator()
														+"PlayResY: 1024"+SubtitleFile.getLineSeparator()+SubtitleFile.getLineSeparator();
	
	private static final String SSA_FIELDS_KEYS_PROPERTY ="ssa.format.fields";
	private static final String SSA_FIELDS_KEYS_DELIM =";";
	private static final String SSA_FIELDS_KEYS_DEFAULT =START_KEY+SSA_FIELDS_KEYS_DELIM+END_KEY+SSA_FIELDS_KEYS_DELIM+TEXT_KEY;
	
	private static final String SSA_END_PART_PROPERTY ="ssa.format.end";
	
	private static final String SSA_END_PART_DEFAULT =""+SubtitleFile.getLineSeparator();
	/** Identifier key for the events section */
	public static final String EVENTS_SECTION_DELIMITER = "[Events]";

	// the events key used by the ssa file:
	private String[] fieldsKey;
	// the first part of the ssa file:
	private String beginSsaFilePart;
	// the last part of the ssa file:
	private String endSsaFilePart;

	/**
	 * Constructor SsaFile.
	 * <br><b>Summary:</b><br>
	 * Constructor of the class.
	 * Beware not to use this file directly, because it does contains no ST.
	 * You will have to fill the list of ST, and save the File first.
	 */
	public SsaFile() {
		super();
		setBeginPart(getDefaultBeginPart());
		setFieldsKey(getDefaultFieldsKey());
		setEndPart(getDefaultEndPart());
	}

	/**
	 * @return The end part to be used by default, when sears created the SSA file.
	 */
	private String getDefaultEndPart() {
		//Look for property, or load default
		return SearsProperties.getProperty(SSA_END_PART_PROPERTY, SSA_END_PART_DEFAULT);
	}

	/**
	 * @return The default keys to be used by ssa , when sears created the SSA file.
	 */
	private String[] getDefaultFieldsKey() {
		String keys = SearsProperties.getProperty(SSA_FIELDS_KEYS_PROPERTY, SSA_FIELDS_KEYS_DEFAULT);
		//Parse these keys
		StringTokenizer stk = new StringTokenizer(keys, SSA_FIELDS_KEYS_DELIM);
		String[] result = new String[stk.countTokens()];
		int index = 0;
		while(stk.hasMoreElements()){
			result[index]=stk.nextToken();
			index++;
		}
		
		return result;
	}

	/**
	 * @return The begin part to be used by default, when sears created the SSA file.
	 */
	private String getDefaultBeginPart() {
		//Look for property, or load default
		return SearsProperties.getProperty(SSA_BEGIN_PART_PROPERTY, SSA_BEGIN_PART_DEFAULT);
	}

	/**
	 * Constructor SsaFile.
	 * <br><b>Summary:</b><br>
	 * Constructor of the class.
	 * @param file            The <b>(File)</b> to open.
	 * @param subtitleList        The <b>(ArrayList)</b> List of subtitles.
	 * @throws FileConversionException 
	 */
	public SsaFile(File file, ArrayList<Subtitle> subtitleList) throws FileConversionException {
		super(file, subtitleList);
	}

	/**
	 * Constructor SsaFile.
	 * <br><b>Summary:</b><br>
	 * Constructor of the class.
	 * @param file                The <b>(String)</b> path to file to open.
	 * @param subtitleList        The <b>(ArrayList)</b> List of subtitles.
	 * @throws FileConversionException 
	 */
	public SsaFile(String file, ArrayList<Subtitle> subtitleList) throws FileConversionException {
		super(file, subtitleList);
	}

	/**
	 * 
	 * @param file
	 * @param subtitleList
	 * @param charset
	 * @throws FileConversionException
	 */
	public SsaFile(File file, ArrayList<Subtitle> subtitleList, String charset) throws FileConversionException {
		super(file, subtitleList, charset);
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.SubtitleFile#getNewInstance()
	 */
	protected SubtitleFile getNewInstance() {
		return new SsaFile();
	}

	protected void setFieldsKey(String[] fieldsKey) {
		this.fieldsKey = fieldsKey;
	}

	protected String[] getFieldsKey() {
		return fieldsKey;
	}

	protected void setBeginPart(String beginPart) {
		this.beginSsaFilePart = beginPart;
	}

	protected void setEndPart(String endPart) {
		this.endSsaFilePart = endPart;
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.SubtitleFile#parse()
	 */
	protected void parse() throws FileConversionException {
		FileConversion pm = null;
		String charset = DEFAULT_CHARSET;
		if( !charset.equals(getCharset()) ) {
			pm = new FileToSsaFile(file, getCharset(), this);
			// try to fill the list of subtitles:
			pm.parse(subtitleList);
		} else {	
			int count = -1;
			while( count < BASIC_CHARSETS.length ) {
				try {
					pm = new FileToSsaFile(file, charset, this);
					// try to fill the list of subtitles:
					pm.parse(subtitleList);
					// parse succeed so, stop the loop:
					count = BASIC_CHARSETS.length;
					setCharset(charset);
				} catch (FileConversionException e) {
					// NOTE FOR DEVELOPER:
					//
					// the catching exception must be tested
					// if the nature of the nature is a basic IO exception
					// it is not necessary to parse again with a different charset
					// the error will occur again					
					count++;
					if( count >= BASIC_CHARSETS.length ) {
						// stop trying to decode:
						throw e;
					} else {
						// change the charset
						charset = BASIC_CHARSETS[count];
					}
				}
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.SubtitleFile#writeToFile(java.io.File)
	 */
	public void writeToFile(File fileToWrite) throws FileConversionException {
		// First is to know which end of line to use.
		// by default use the linux one.
		String lineSeparator = getLineSeparator();
		try {
			// keep the initial encoding:
			BufferedWriter out = new BufferedWriter(
					new OutputStreamWriter(
							new FileOutputStream(fileToWrite), super.getCharset()));

			// **************************
			// we write the first part of ssa file:
			out.write(beginSsaFilePart);

			// **************************
			// we write the key [Events]:
			out.write(EVENTS_SECTION_DELIMITER + lineSeparator);

			// **************************
			// we write the format line:
			String str = FORMAT_KEY + " ";				
			for( int i=0;i<fieldsKey.length - 1;i++ ) {
				str = str + fieldsKey[i] + ", ";
			}
			str = str + fieldsKey[fieldsKey.length - 1] + lineSeparator;			
			out.write(str);

			// **************************
			// we write all subtitles:
			for (Subtitle subtitle : subtitleList) {
				out.write(DIALOGUE_KEY + " ");
				for( int i=0;i<fieldsKey.length;i++ ) {
					if( fieldsKey[i].contentEquals("Start") ) {
						out.write(timeToString(subtitle.getStartDate()) + ",");
					} else if( fieldsKey[i].contentEquals("End") ) {
						out.write(timeToString(subtitle.getEndDate()) + ",");
					} else if( fieldsKey[i].contentEquals("Text") ) {
						//remove all line separator
						out.write(subtitle.getSubtitle().replace(Utils.LINE_SEPARATOR, " "));
						out.write(subtitle.getSubtitle().replace(getLineSeparator(), " "));
						//Check wether subtitle has an end of line.
						if( !subtitle.getSubtitle().endsWith(lineSeparator) ){
							//If it doesn't have one, put one.
							//out.write(lineSeparator);
						}
					} else {
						if(subtitle instanceof SsaSubtitle){
							out.write(((SsaSubtitle)subtitle).getEntrie(fieldsKey[i]) + ",");
						}else{
							//We are trying to write a non SSA subtitle to a SSA file, print empty string
							out.write(" " + ",");
						}
					}				
				}
				out.write(endSsaFilePart);
			}

			out.close();
			//indicate file is good.
			fileChanged = false;
		} catch (IOException e) {
			throw FileConversionException.getAccessException(
					FileConversionException.WRITE_ACCESS, file);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.SubtitleFile#writeToTemporaryFile()
	 */
	public void writeToTemporaryFile() {
		try {
			// Create the temporary subtitle file
			if (temporaryFile == null) {
				temporaryFile = java.io.File.createTempFile(
						extension().toUpperCase() + "Subtitle", null);
				temporaryFile.deleteOnExit();
			}
			// Store to the temporary file
			boolean oldFileChangedStatus = fileChanged;
			writeToFile(temporaryFile);
			// Restore the old file changed value
			fileChanged = oldFileChangedStatus;
		} catch (IOException e) {
			Trace.trace("Error while writing temporary " + extension().toUpperCase() + " file !",
					Trace.ERROR_PRIORITY);
			Trace.trace(e.getMessage(), Trace.ERROR_PRIORITY);
		}
	}

	/**
	 * Method stringToTime.
	 * <br><b>Summary:</b><br>
	 * Return the number of miliseconds that correspond to the given String time representation.
	 * @param time              The string ssa time representation.
	 * @return <b>(int)</b>     The corresponding number of miliseconds.
	 */
	public static int stringToTime(String time) throws NumberFormatException{
		// we add a 0 for time conversion compatibility:
		return SubtitleFile.stringToTime(time + 0);
	}

	/**
	 * Method timeToString.
	 * <br><b>Summary:</b><br>
	 * This method transform a number of milliseconds in a string representation.
	 * @param milliseconds               The number of milliseconds to transform
	 * @return  <b>(String)</b>     The corresponding String representation of the number of milliseconds.
	 */
	public static String timeToString(int milliseconds){
		String result = SubtitleFile.timeToString(milliseconds);
		// 00:00:00,000 --> 0:00:00.00
		// we erase the first and last number (no rounded):
		result = result.substring(1, result.length()-1);	
		// and replace comma by point:
		result = result.replace(",", ".");
		return result;	 
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.SubtitleFile#extension()
	 */
	public String extension() {
		return "ssa";
	}

	// there's maybe a problem with ass file:

	/*
	 * (non-Javadoc)
	 * @see sears.file.SubtitleFile#split(java.io.File[], int, int)
	 */
	public SubtitleFile[] split(File[] destinationFiles, int subtitleIndex, int secondPartDelay) {
		//The result of the method
		SubtitleFile[] result = new SubtitleFile[2];
		//Construct first part Subtitle File.
		result[0] = getNewInstance();
		//set its file.
		result[0].setFile(destinationFiles[0]);
		//and second part.
		result[1] = getNewInstance();
		//set its file.
		result[1].setFile(destinationFiles[1]);
		//Fill in the subtitle Files.
		int index = 0;
		//by parsing current STs list, and add to one or other file.
		for (Subtitle currentSubtitle : subtitleList) {
			//If number is before limit, add to first part.
			if (index < subtitleIndex) {
				result[0].addSubtitle(new SsaSubtitle(currentSubtitle));
			} else {
				//else, add to the second part.
				result[1].addSubtitle(new SsaSubtitle(currentSubtitle), true);
			}
			index++;
		}

		//Apply delay.
		if(secondPartDelay >= 0){
			//Delay second part, so first ST is at time 0
			result[1].shiftToZero();
			result[1].delay(secondPartDelay);
		}
		//return the result;
		return result;
	}
}

/**
 * Parsing Ssa subtitle 
 */
class FileToSsaFile extends FileConversion {

	// there's no subtitle number store in a sub file
	// so a meter is necessary:
	private int subtitleCount;

	private SsaFile ssaFile;

	/**
	 * Constructs a new instance of <tt>SsaFileConversion</tt>
	 * @param file		the file to convert
	 * @param charset	the charset used for the conversion
	 * @param ssaFile	the resulting ssaFile, use to fill some part of <tt>file</tt>
	 * @throws FileConversionException
	 */
	public FileToSsaFile(File file, String charset, SsaFile ssaFile) throws FileConversionException {
		super(file, charset);
		if( ssaFile == null ) {
			throw new NullPointerException("ssaFile cannot be null");
		}
		this.ssaFile = ssaFile;
		subtitleCount = 0;
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.FileConversion#parse(java.util.ArrayList)
	 */
	public void parse(ArrayList<Subtitle> subtitleList) throws FileConversionException {
		ssaFile.setBeginPart(getBeginPart());
		ssaFile.setFieldsKey(getFieldsKey());
		super.parse(subtitleList);
		ssaFile.setEndPart(getEndPart());
	}

	// Dialogue: Marked=0,0:00:20.03,0:00:21.03,Default,NTP,0000,0000,0000,!Effect,Tales of Earthsea
	protected Subtitle getSubtitle(String line) throws FileConversionException {
		SsaSubtitle ssaSubtitle = null;
		String str = "";
		if( line != null && line.trim().length() != 0 ) {
			if( !line.startsWith(SsaFile.DIALOGUE_KEY)) {
				throw FileConversionException.getMalformedSubtitleFileException(
						FileConversionException.MALFORMED_SUBTITLE_FILE, file, lineCount, line);
			} else {
				// we remove string array "Dialogue:"
				str = line.replace(SsaFile.DIALOGUE_KEY, "");
			}

			// ********
			// NUMBER *
			// ********
			// we create an instance of the SubtitleStyle class with the 
			// specified number of subtitle:
			ssaSubtitle = new SsaSubtitle(++subtitleCount);


			// ********
			// FIELDS *
			// ********
			// it's needed to prepare line before tokenize it
			// if sub-array ',,' exists is not considered like a token !
			// so we put a space instead of nothing...
			str = str.replace(",,", ", ,");
			// we create a tokenizer:
			StringTokenizer stk = new StringTokenizer(str, ",");
			// and stores the elements:
			String aToken = "";
			String[] fieldsKey = ssaFile.getFieldsKey();
			for(int i=0; i<fieldsKey.length - 1; i++){
				aToken = stk.nextToken();
				// if token does not represent the subtitle array text,
				// we remove all spaces:						
				ssaSubtitle.putEntrie(fieldsKey[i], aToken.replace(" ", ""));
			}

			// ******
			// TEXT *
			// ******
			// a precaution if the subtitle field is empty:
			if( stk.hasMoreTokens() ) {
				aToken = stk.nextToken();
				while( stk.hasMoreTokens() ) {
					aToken = aToken + "," + stk.nextToken();
				}
			} else {
				// empty string 
				aToken = "";
			}			
			ssaSubtitle.putEntrie(fieldsKey[fieldsKey.length - 1], aToken);

			// *************
			// "TIME LINE" *
			// *************
			String startDate = ssaSubtitle.getEntrie("Start");
			String endDate = ssaSubtitle.getEntrie("End");
			if( startDate == null && endDate == null ) {
				throw FileConversionException.getMalformedSubtitleFileException(
						FileConversionException.NO_SUBTITLE_TIME, file, lineCount, line);
			}

			// ********************
			// START AND END TIME *
			// ********************
			try {
				// START TIME 
				try {
					ssaSubtitle.putEntrie("Start", String.valueOf((
							SsaFile.stringToTime(startDate))));
				} catch (NumberFormatException e) {
					// time string is not formatted like a time
					throw FileConversionException.getMalformedSubtitleFileException(
							FileConversionException.MALFORMED_START_TIME, file, lineCount, line);
				} 

				// END TIME
				try {
					ssaSubtitle.putEntrie("End", String.valueOf((
							SsaFile.stringToTime(ssaSubtitle.getEntrie("End")))));
				} catch (NumberFormatException e) {
					// time string is not formatted like a time
					throw FileConversionException.getMalformedSubtitleFileException(
							FileConversionException.MALFORMED_END_TIME, file, lineCount, line);
				} 
			} catch (NullPointerException e) {
				// Start or end time doesn't not exists in the fields key
				throw FileConversionException.getMalformedSubtitleFileException(
						FileConversionException.MALFORMED_TIME_LINE, file, lineCount, line);
			}
		}
		return ssaSubtitle;
	}
	
	/**
	 * Gets the begin part of the file, until the events section delimiter
	 * <br>The reader state is on the events section delimiter line 
	 * @return							the begin part as a <tt>String</tt>
	 * @throws FileConversionException	if an error occurs 
	 * @see SsaFile#EVENTS_SECTION_DELIMITER
	 */
	private String getBeginPart() throws FileConversionException {
		String str = getTheNextNonEmptyLine();
		if( str == null ) {
			throw FileConversionException.getMalformedSubtitleFileException(
					FileConversionException.EMPTY_SUBTITLE_FILE, file);
		} // else	

		try {
			String line = str;
			while( !line.startsWith(SsaFile.EVENTS_SECTION_DELIMITER) ) {
				str += line + Utils.LINE_SEPARATOR;
				line = readLine();
			}
		} catch (NullPointerException e) {
			// USE EXCEPTION (Rare)
			// str == null we reach end of file without founded the EVENTS_SECTION_DELIMITER
			// that means the subtitle is malformed, there's no subtitle part
			throw FileConversionException.getMalformedSubtitleFileException(
					FileConversionException.UNEXPECTED_END_OF_FILE, file);
		}

		return str;
		// Reader is on the line which contains the EVENTS_SECTION_DELIMITER
	}

	private String getEndPart() throws FileConversionException {
		String str = "";
		// while there's line to be read:
		String line = "";
		while( line != null ) {
			// we read line:
			str += line + Utils.LINE_SEPARATOR;
			line = readLine();
		}	
		return str;
	}

	// Format: Marked, Start, End, Style, Name, MarginL, MarginR, MarginV, Effect, Text
	private String[] getFieldsKey() throws FileConversionException {
		// Reader positioned on the line which contains the EVENTS_SECTION_DELIMITER
		String initialLine = getTheNextNonEmptyLine();
		if( initialLine == null ) {
			// Events section do not contains anything 
			throw FileConversionException.getMalformedSubtitleFileException(
					FileConversionException.UNEXPECTED_END_OF_FILE, file);
		}

		// ******************
		// STRING "FORMAT:"	*
		// ******************
		String str = "";
		if( initialLine.startsWith(SsaFile.FORMAT_KEY) ) {
			str = initialLine.replace(SsaFile.FORMAT_KEY, "");	
		} else {
			// the line is malformed:
			FileConversionException.getMalformedSubtitleFileException(
					FileConversionException.MALFORMED_SUBTITLE_FILE, file, lineCount, initialLine);
		}		

		// ******************
		// 'TOKENIZE' LINE: *
		// ******************

		// TEST THE VALIDITY OF THE LINE:
		// remove all spaces present in the string:
		str = str.replaceAll(" ", "");
		// we create a tokenizer:
		StringTokenizer stk = new StringTokenizer(str, ",");
		int countTokens = stk.countTokens();
		if( countTokens == 0 ) {
			// there's no field and so no text field:
			FileConversionException.getMalformedSubtitleFileException(
					FileConversionException.MALFORMED_SUBTITLE_FILE, file, lineCount, initialLine);
		}

		// PARSING LINE		
		int linePosition = 0;
		// else there's at least one field			
		String[] fieldsKey = new String[countTokens];
		//stores the first element:
		str = stk.nextToken();				
		fieldsKey[linePosition++] = str;
		// and finally stores the others elements:
		while( stk.hasMoreElements() ) {
			str = stk.nextToken(); 
			fieldsKey[linePosition] = str;
			linePosition++;
		}

		return fieldsKey;
	}
}