/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////

package sears.file;

import java.util.HashMap;

/**
 * this class represents a ssa subtitle.
 */
public class SsaSubtitle extends Subtitle {

	/** keys for 'events' section */
	public static final String[] eventsDictionnaryKeys = {
		"Marked","Layer","Start","End","Style","Name",
		"MarginL","MarginR","MarginV","Effect","Karaoke",
		"Text","Comment","Picture","Sound","Movie",
		"Command"
	};	

	private HashMap<String,String> dictionnary = null;

	/**
	 * Constructor
	 * @param subtitleNumber
	 */
	public SsaSubtitle(int subtitleNumber) {
		super.setNumber(subtitleNumber);
		super.anchor(-1);
		constructAnEmptyDictionnary();		
	}

	/**
     * Constructor Subtitle.
     * Copy another subtitle.
     * @param aSubtitle  The <b>Subtitle</b> to copy.
     */
	public SsaSubtitle(Subtitle aSubtitle) {
		this(aSubtitle.getNumber());
		
		if(aSubtitle instanceof SsaSubtitle) {
			for(int i=0;i<eventsDictionnaryKeys.length;i++) {		
				this.putEntrie(eventsDictionnaryKeys[i], 
						((SsaSubtitle)aSubtitle).getEntrie(eventsDictionnaryKeys[i]));
			}
		} else {
			// tempory...
			System.out.println("fresh and copy needed");
		}
	}

	/**
	 * this method constructs an empty dictionnary with the events keys
	 */
	private void constructAnEmptyDictionnary() {
		dictionnary = new HashMap<String,String>(eventsDictionnaryKeys.length);
		for(int i=0;i<eventsDictionnaryKeys.length;i++) {
			dictionnary.put(eventsDictionnaryKeys[i], null);
		}
	}

	/**
	 * this method allows to change a value of a key.
	 * No add-on is possible, the key must be already exist...
	 * @param key		the existing key
	 * @param value		the value who wants to associated to the key
	 * @return			true if the key exists, false if not
	 */
	public boolean putEntrie(String key, String value) {
		boolean result = false;
		// just in case:
		if(dictionnary != null) {		
			result = dictionnary.containsKey(key);
			if(result == true) {
				dictionnary.put(key, value);
			}	
		}
		return result;
	}

	/**
	 * this method returns the value associated to a key in a dictionnary.
	 * @param key		the existing key
	 * @return			the associated value, if the value is null, null is returned.
	 * 					<br>There's no way to know if the key does not exist in the dictionnary
	 */
	public String getEntrie(String key) {
		String result = null;
		// just in case:
		if(dictionnary != null) {			
			if(dictionnary.containsKey(key)) {
				result = dictionnary.get(key);
			}	
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.Subtitle#getEndDate()
	 */
	public int getEndDate() {
		return Integer.parseInt(getEntrie("End"));
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.Subtitle#setEndDate(int)
	 */
	public void setEndDate(int endDate) {
		putEntrie("End", String.valueOf(endDate));
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.Subtitle#getStartDate()
	 */
	public int getStartDate() {
		return Integer.parseInt(getEntrie("Start"));
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.Subtitle#setStartDate(int)
	 */
	public void setStartDate(int startDate) {
		putEntrie("Start", String.valueOf(startDate));
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.Subtitle#getSubtitle()
	 */
	public String getSubtitle() {
		return getEntrie("Text");
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.Subtitle#setSubtitle(java.lang.String)
	 */
	public void setSubtitle(String subtitle) {
		putEntrie("Text", subtitle);
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.Subtitle#delay(int)
	 */
	public void delay(int delay) {
		setStartDate(getStartDate() + delay);
		setEndDate(getEndDate() + delay);
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.Subtitle#accentRemove()
	 */
	public void accentRemove() {
		// we store subtitle:
		super.setSubtitle(getSubtitle());
		// aplly the accent remove method:
		super.accentRemove();
		// and update dictionnary:
		setSubtitle(super.getSubtitle());
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.Subtitle#htmlRemove()
	 */
	public void htmlRemove() {
		// we store subtitle:
		super.setSubtitle(getSubtitle());
		// aplly the html remove method:
		super.htmlRemove();
		// and update dictionnary:
		setSubtitle(super.getSubtitle());
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.Subtitle#compareTo(sears.file.Subtitle)
	 */
	public int compareTo(Subtitle subtitle) {
		// we store start date:
		super.setStartDate(getStartDate());		 
		return super.compareTo(subtitle);
	}
}