/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

//some suggestions about this class: floriaen@gmail.com

package sears.file;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import sears.file.exception.io.FileConversionException;
import sears.tools.Trace;
import sears.tools.Utils;

/**
 * Class SubFile.
 * <br><b>Summary:</b><br>
 * This class represents a sub subtitle file.
 * Specialize the SubtitleFile for sub type subtitles.
 */

/*
 * { xxx } xxx represents the frame apparition/disparition on video
 * xxx/fps is the time in seconds of the apparition/disparition of subtitle on screen
 */
public class SubFile extends SubtitleFile {

	/** the line separator in the sub subtitle text: '|' */
	protected static final String LINE_SEPARATOR = "|";

	/** The time separator, used by <tt>StrinkTokenozer</tt> object to gets back start and end time*/
	protected static final String TIME_SEPARATOR = "{}";

	// ^ begin of line
	// looking for '\\{\\d+\\}' ( '\d' a number,  '\d+' 1 or more number )
	// {2} 2 time
	/** pattern that represents sub subtitle time line, a string that begin by: {number}{number}*/
	protected static final String TIME_LINE_PATTERN = "^(\\{\\d+\\}){2}";
	
	
	/** Standards framerate */
	protected static final double MOVIE_FRAMERATE = 23.976;
	//protected static final double ??_FRAMERATE = 24;
	protected static final double PAL_FRAMERATE = 25;
	protected static final double NTSC_FRAMERATE = 29.97;
	//protected static final double ??_FRAMERATE = 30;
	
	/* the framerate used to calculate subtitle time */
	private double usedFramerate;

	/**
	 * Constructor SubFile.
	 * <br><b>Summary:</b><br>
	 * Constructor of the class.
	 * Beware not to use this file directly, because it does contains no ST.
	 * You will have to fill the list of ST, and save the File first.
	 */
	public SubFile(){
		super();
		usedFramerate = PAL_FRAMERATE;
	}

	/**
	 * Constructor SubFile.
	 * <br><b>Summary:</b><br>
	 * Constructor of the class.
	 * @param file            The <b>(File)</b> to open.
	 * @param subtitleList        The <b>(ArrayList)</b> List of subtitles.
	 * @throws FileConversionException
	 */
	public SubFile(File file, ArrayList<Subtitle> subtitleList) throws FileConversionException {
		super(file, subtitleList);
		usedFramerate = PAL_FRAMERATE;
	}

	/**
	 * Constructor SubFile.
	 * <br><b>Summary:</b><br>
	 * Constructor of the class.
	 * @param file                The <b>(String)</b> path to file to open.
	 * @param subtitleList        The <b>(ArrayList)</b> List of subtitles.
	 * @throws FileConversionException
	 */
	public SubFile(String file, ArrayList<Subtitle> subtitleList) throws FileConversionException {
		super(file, subtitleList);
		usedFramerate = PAL_FRAMERATE;
	}

	public SubFile(File file, ArrayList<Subtitle> subtitleList, String charset) throws FileConversionException {
		super(file, subtitleList, charset);
		usedFramerate = PAL_FRAMERATE;
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.SubtitleFile#extension()
	 */
	public String extension() {
		return "sub";
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.SubtitleFile#getNewInstance()
	 */
	protected SubtitleFile getNewInstance() {
		return new SubFile();
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.SubtitleFile#parse()
	 */
	protected void parse() throws FileConversionException {
		// AUTOMATION OF CHARSET
		FileConversion pm = null;
		String charset = DEFAULT_CHARSET;
		if( !charset.equals(getCharset()) ) {
			pm = new FileToSubFile(file, getCharset());
			// try to fill the list of subtitles:
			pm.parse(subtitleList);
		} else {	
			int count = -1;
			while( count < BASIC_CHARSETS.length ) {
				try {
					pm = new FileToSubFile(file, charset);
					// try to fill the list of subtitles:
					pm.parse(subtitleList);
					// parse succeed so, stop the loop:
					count = BASIC_CHARSETS.length;
					setCharset(charset);
				} catch (FileConversionException e) {
					// NOTE FOR DEVELOPER:
					//
					// the catching exception must be tested
					// if the nature of the nature is a basic IO exception
					// it is not necessary to parse again with a different charset
					// the error will occur again
					count++;
					if( count >= BASIC_CHARSETS.length ) {
						// stop trying to decode:
						throw e;
					} else {
						// use another charset to parsing the file
						charset = BASIC_CHARSETS[count];
					}
				}
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.SubtitleFile#writeToFile(java.io.File)
	 */
	public void writeToFile(File fileToWrite) throws FileConversionException {
		//First is to know which end of line to use.
		//by default use the linux one.
		String endOfLine = getLineSeparator();
		try {
			// keep the initial encoding:
			BufferedWriter out = new BufferedWriter(
					new OutputStreamWriter(
							new FileOutputStream(fileToWrite), super.getCharset()));

			Iterator<Subtitle> subtitles = subtitleList.iterator();
			while (subtitles.hasNext()) {
				Subtitle subtitle = subtitles.next();

				// format START DATE and write it:
				String date = String.valueOf(subtitle.getStartDate());
			/*	int dateLength = date.length();
				if( dateLength > 2 ) {
					date = date.substring(0, date.length()-2); 
				}*/
				String startFrame = millisecondsToFrame( date, usedFramerate );
				out.write("{" + startFrame + "}");

				// format END DATE and write it:
				date = String.valueOf(subtitle.getEndDate());
				//date = date.substring(0, date.length()-2); 
				String endFrame = millisecondsToFrame( date, usedFramerate );
				out.write("{" + endFrame + "}");

				// SUBTITLE:
				String subtitleText = subtitle.getSubtitle();
				// remove the last line separator
				if( subtitleText.endsWith(Utils.LINE_SEPARATOR) ) {
					int length = subtitleText.length();
					subtitleText = subtitleText.substring(0, length - Utils.LINE_SEPARATOR.length());
				}				
				out.write(subtitleText.replace(Utils.LINE_SEPARATOR, SubFile.LINE_SEPARATOR));
				// write the last line separator
				out.write(endOfLine);

			}
			out.close();
			//indicate file is good.
			fileChanged = false;
		} catch (IOException e) {
			// FileNotFoundException
			// IOException
			throw FileConversionException.getAccessException(
					FileConversionException.WRITE_ACCESS, file);
		} catch( SecurityException e) {
			throw FileConversionException.getAccessException(
					FileConversionException.WRITE_ACCESS, file);
		}
	}
	
	private String millisecondsToFrame( String milliseconds, double frameRate ) {
		double convertedMilliseconds = 0;
		try {
			convertedMilliseconds = Double.parseDouble(milliseconds);
		} catch ( NumberFormatException e ) {
			Trace.trace(milliseconds+", invalid integer", Trace.ERROR_PRIORITY);
			// stop the processus ---> TODO
		}
		double frameDouble = ( (convertedMilliseconds /1000) * frameRate );
		int frameInteger = ( int ) frameDouble;
		//System.out.println( "DOUBLE: "+frameDouble );
		//System.out.println( convertedMilliseconds+" ms, "+frameInteger+" frame, FRAMERATE="+frameRate);
		return String.valueOf(frameInteger);
		
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.SubtitleFile#writeToTemporaryFile()
	 */
	public void writeToTemporaryFile() {
		try {
			// Create the temporary subtitle file
			if (temporaryFile == null) {
				temporaryFile = File.createTempFile("SUBSubtitle", null);
				temporaryFile.deleteOnExit();
			}
			// Store to the temporary file
			boolean oldFileChangedStatus = fileChanged;
			writeToFile(temporaryFile);
			// Restore the old file changed value
			fileChanged = oldFileChangedStatus;
		} catch (IOException e) {
			Trace.trace("Error while writing temporary SUB file !",
					Trace.ERROR_PRIORITY);
			Trace.trace(e.getMessage(), Trace.ERROR_PRIORITY);
		}
	}
}

class FileToSubFile extends FileConversion {

	// there's no subtitle number store in a sub file
	// so a meter is necessary:
	private int subtitleCount;

	public FileToSubFile(File file, String charset) throws FileConversionException {
		super(file, charset);
		subtitleCount = 0;
	}
	
	/*
	 * see bug #1901242
	 * sub frame: 1114 / (frameRate) fps ---> time on screen
	 * { xxx } xxx represents the frame apparition on video
	 * 
	 */
	private int convertFrameToMilliseconds(double subFrame, double frameRate ) {
		return (int) ( (subFrame/frameRate)*1000 );	
	}

	// AUTOMATIC CORRECTION

	//
	// time: No double, no long
	// subtitle text could be empty
	protected Subtitle getSubtitle(String line)	throws FileConversionException {
		Subtitle subtitle = null;		
		if( line != null) {	
			boolean endOfFile = false;
			// if the line is empty, try to read subtitle until the next non empty line
			if( line.trim().length() == 0 ) {
				// gets the next non empty line
				// could be null if it is the end of file
				line = super.getTheNextNonEmptyLine();
				if( line == null ) {
					// END OF FILE
					// WE STOP PARSING FILE
					endOfFile = true;					
				}
			}

			// *************
			// PARSING JOB *
			// *************
			// Line is non null and non empty,
			// begins parsing the line.
			//
			// A valid SUB subtitle line looks like: 
			// "{1286}{1329}- There you are."
			//
			if( !endOfFile ) {
				String timeLine = null;

				int startTime = 0;
				int endTime = 0;
				String text = null;			

				// ********************
				// GETS THE TIME LINE *
				// ********************
				Pattern pattern = Pattern.compile(SubFile.TIME_LINE_PATTERN);
				Matcher matcher = pattern.matcher(line);
				if( matcher.find() ) {
					// time line looks like : {number}{number}
					timeLine = matcher.group();
				} else {
					// STOP PARSING FILE,
					// there's no time line in this line, or it's corrupt
					// like {89UYh0}{89 for example
					//
					// ---------------------------------------------------------------------
					// NOTE FOR DEVELOPER:
					// maybe we can try to correct the time line:
					//  - if a '{' or '}' is missing, we can try to add it
					//  - in the case that's there's letter in time part, we could remove it.
					// ---------------------------------------------------------------------  
					throw FileConversionException.getMalformedSubtitleFileException(
							FileConversionException.MALFORMED_TIME_LINE, file, lineCount, line);
				}


				// *************************
				// GETS START AND END TIME *
				// *************************
				StringTokenizer stk = new StringTokenizer(timeLine, SubFile.TIME_SEPARATOR);
				// There's two tokens, timeLine is looks like {7879}{7890}
				// The two tokens are integer, if there's too long (double or long) 
				// an exception is throws
				//
				// ---------------------------------------------------------------------
				// NOTE FOR DEVELOPER:
				// maybe we can accept long and double, and so the parse could continue
				// ---------------------------------------------------------------------
				try {
					double startFrame = Double.parseDouble(stk.nextToken());
					startTime = convertFrameToMilliseconds( startFrame, SubFile.PAL_FRAMERATE  );
					
					//System.out.println( startFrame +" frame-->"+startTime+" milliseconds");
					
				} catch( NumberFormatException e ) {
					// stk.nextToken() is long or double and non null
					throw FileConversionException.getMalformedSubtitleFileException(
							FileConversionException.MALFORMED_START_TIME, file, lineCount, line);
				}

				try {
					double endFrame = Double.parseDouble(stk.nextToken());
					endTime = convertFrameToMilliseconds( endFrame, SubFile.PAL_FRAMERATE  );
				} catch( NumberFormatException e ) {
					// stk.nextToken() is long or double and non null
					throw FileConversionException.getMalformedSubtitleFileException(
							FileConversionException.MALFORMED_END_TIME, file, lineCount, line);
				}

				// ********************
				// GETS SUBTITLE TEXT *
				// ********************
				// remove the time line from the line
				// the line is non null so subtitle could not be null
				// it could be an empty string, accept it
				text = matcher.replaceAll("");
				// IMPORTANT POINT:
				// replaces all the sub file specific line separator by the generic line separator
				text = text.replace(SubFile.LINE_SEPARATOR, Utils.LINE_SEPARATOR);

				// *****************
				// CREATE SUBTITLE *
				// *****************
				subtitle = new Subtitle(++subtitleCount, startTime, endTime, text);
			} // else newSubtitle is return as null
		} // else newSubtitle is return as null
		return subtitle;
	}
}