package sears.file.exception.io;

import java.io.File;
import java.io.IOException;

import sears.file.FileSystemAccess;
import sears.tools.Utils;

/**
 * A <tt>FileConversionException</tt> is throw when an error occurs during a file access
 * and conversion to subtitle file
 * <br>This class provides <tt>static</tt> methods to create generic objects using
 * the class constants.
 * <p>
 * Two distinct categories of constants:
 * <br>- file access exception ( read and write access )
 * <br>- malformed subtitle file exception
 * </p>
 */
public abstract class FileConversionException extends IOException {

	/** Default Serial UID */
	private static final long serialVersionUID = 1L;
	// FILE ACCESS EXCEPTION :
	// +++++++++++++++++++++
	//
	/** FILE ACCESS EXCEPTION: Read access exception */
	public static final int READ_ACCESS 				= 0;
	/** FILE ACCESS EXCEPTION: Write access exception */
	public static final int WRITE_ACCESS 				= 1;
	/** FILE ACCESS EXCEPTION: unsupported file size exception */
	public static final int UNSUPPORTED_FILE_SIZE 		= 2;

	// MALFORMED SUBTITLE FILE EXCEPTION :
	// +++++++++++++++++++++++++++++++++
	//
	// + NUMBER - MALFORMED SUBTITLE FILE EXCEPTION
	/** MALFORMED SUBTITLE FILE EXCEPTION: no subtitle number exception */
	public static final int NO_SUBTITLE_NUMBER 			= 3;
	/** MALFORMED SUBTITLE FILE EXCEPTION: malformed subtitle number exception */
	public static final int MALFORMED_SUBTITLE_NUMBER 	= 4;

	// + TIME - MALFORMED SUBTITLE FILE EXCEPTION
	/** MALFORMED SUBTITLE FILE EXCEPTION: no subtitle time exception */
	public static final int NO_SUBTITLE_TIME			= 5;
	/** MALFORMED SUBTITLE FILE EXCEPTION: malformed subtitle time line exception */
	public static final int MALFORMED_TIME_LINE 		= 6;
	/** MALFORMED SUBTITLE FILE EXCEPTION: malformed subtitle start time exception */
	public static final int MALFORMED_START_TIME 		= 7;
	/** MALFORMED SUBTITLE FILE EXCEPTION: malformed subtitle end time exception */
	public static final int MALFORMED_END_TIME 			= 8;

	// + TEXT - MALFORMED SUBTITLE FILE EXCEPTION
	/** MALFORMED SUBTITLE FILE EXCEPTION: no subtitle text exception */
	public static final int NO_SUBTITLE_TEXT 			= 9;
	/** MALFORMED SUBTITLE FILE EXCEPTION: malformed subtitle text exception */
	public static final int MALFORMED_SUBTITLE_TEXT 	= 10;

	// + GENERIC MALFORMED SUBTITLE FILE EXCEPTION
	/** MALFORMED SUBTITLE FILE EXCEPTION: generic subtitle file exception */
	public static final int MALFORMED_SUBTITLE_FILE		= 11;

	// + SPECIAL MALFORMED SUBTITLE FILE EXCEPTION
	/** MALFORMED SUBTITLE FILE EXCEPTION: unexpected end of file exception */
	public static final int UNEXPECTED_END_OF_FILE 		= 12;
	/** MALFORMED SUBTITLE FILE EXCEPTION: empty subtitle file exception */
	public static final int EMPTY_SUBTITLE_FILE			= 13;

	/**
	 * Returns the error message
	 * @return the error message
	 */
	public abstract String getMessage();

	/**
	 * Returns the detail error message
	 * @return the detail error message
	 */
	public abstract String getDetail();

	/**
	 * Returns a <tt>boolean</tt> which indicates if the error could have
	 * appeared because of a bad encoding/decoding
	 * @return 	true if the error could be appear because a bad encoding/decoding
	 * 			false if not
	 */
	public abstract boolean couldBeABadEncoding();

	// returns a formatted message by analyze <tt>file</tt> parameter
	private static String fileDetail(File file) {
		String detail = "";
		try {
			detail = file.getPath() + Utils.LINE_SEPARATOR;
		} catch( NullPointerException e ) {
			detail = "";
		}
		return detail;
	}

	// READ ERROR
	// returns a formatted error message by testing <tt>file</tt> parameter
	private static String iOExceptionDetailWhenReading(File file) {
		String message = null;
		if( file != null ) {
			try {
				if( file.exists() ) {
					if( !file.canRead() ) {
						message = "File read access is denied";
					} else {
						message = "Unknow error, the file seems to be readable";
					}
				} else {
					message = "File does not exist";
				}
			} catch (SecurityException e) {
				message = "There's a security restriction on this file";
			}
		} else {
			message = "Unknow error";
		}
		return message;
	}

	// WRITE ERROR
	// returns a formatted error message by testing <tt>file</tt> parameter
	private static String iOExceptionDetailWhenWriting(File file) {
		String message = null;
		if( file != null ) {
			try {
				if( file.exists() ) {
					if( !file.canWrite() ) {
						message = "File write access is denied";
					} else {
						message = "Unknow error, the file seems to be writable";
					}
				} else {
					message = "File does not exist";
				}
			} catch (SecurityException e) {
				message = "There's a security restriction on this file";
			}
		} else {
			message = "Unknow error";
		}
		return message;
	}

	// LENGTH ERROR
	// returns a formatted error message by testing <tt>file</tt> parameter
	private static String getFileLengthMessage(File file) {
		String message = "";
		try {
			message = file.length() + " octets is too big"
			+ Utils.LINE_SEPARATOR 
			+ "SEARS do not support file bigger than " 
			+ FileSystemAccess.MAX_SUBTITLE_FILE_SIZE + " octets";	
		} catch (SecurityException e) {
			message = "SEARS do not support file bigger than " 
				+ FileSystemAccess.MAX_SUBTITLE_FILE_SIZE + " octets";	
		} catch (NullPointerException e) {
			// DEVELOPPER:
			// ????
			// file is null !!!!
			message = "Unknow error"; 
		}
		return message;
	}

	/**
	 * Get a new <tt>FileConversionException</tt> instance
	 * @param type	<tt>READ_ACCESS</tt>, <tt>WRITE_ACCESS</tt>, <tt>UNSUPPORTED_FILE_SIZE</tt>
	 * @param file	the file where the error is encountered
	 * @return		a new read access exception
	 */
	public static FileConversionException getAccessException(int type, File file) {
		String message = "";
		String detail = "";
		switch ( type ) {
		case READ_ACCESS:
			message = "Read access error";
			detail 	= fileDetail(file) + iOExceptionDetailWhenReading(file) + ", "
			+ Utils.LINE_SEPARATOR + "please checks your authorisation on this file";
			break;
		case WRITE_ACCESS:
			message = "Write access error";
			detail 	= fileDetail(file) + iOExceptionDetailWhenWriting(file) + ", "
			+ Utils.LINE_SEPARATOR + "please checks your authorisation on this file";
			break;
		case UNSUPPORTED_FILE_SIZE:
			message = "Unsupported file size";
			detail 	= fileDetail(file) + getFileLengthMessage(file);
			break;
		default:
			throw new IllegalArgumentException(type + " is not a valid identifier for an exception type");
		}
		return new Bean(message, detail, false);
	}

	/**
	 * Get a new <tt>FileConversionException</tt> instance
	 * @param type	<tt>UNEXPECTED_END_OF_FILE</tt>, <tt>EMPTY_SUBTITLE_FILE</tt>
	 * @param file	the file on which the error occurs
	 * @return		a new malformed subtitle file exception
	 */
	public static FileConversionException getMalformedSubtitleFileException(int type, File file) {
		String message = "";
		String detail = "";
		switch ( type ) {
		case UNEXPECTED_END_OF_FILE:
			message = "Subtitle file seems to be corrupted";
			detail 	= fileDetail(file) + "unexpected end of file";
			break;
		case EMPTY_SUBTITLE_FILE:
			message = "file is empty";
			detail 	= fileDetail(file) + "Sears can't read empty subtitle file";
			break;
		default:
			throw new IllegalArgumentException(type + " is not a valid identifier for an exception type");
		}
		return new Bean(message, detail, false);
	}

	/**
	 * Get a new <tt>FileConversionException</tt> instance
	 * @param type			<tt>MALFORMED_SUBTITLE_NUMBER <br>NO_SUBTITLE_TIME
	 *						<br>MALFORMED_TIME_LINE<br>MALFORMED_START_TIME
	 *						<br>MALFORMED_END_TIME<br>NO_SUBTITLE_TEXT
	 *						<br>MALFORMED_SUBTITLE_TEXT<br>MALFORMED_SUBTITLE_FILE</tt>
	 * @param file			the file on which the error occurrs
	 * @param lineNumber	the line number where the error is encountered
	 * @param line			the line where the error is encountered
	 * @return				a new <tt>FileConversionException</tt> instance
	 */
	public static FileConversionException getMalformedSubtitleFileException(int type, File file, int lineNumber, String line) {
		String message = "Subtitle file seems to be corrupted";
		String detail = "";
		switch ( type ) {
		case NO_SUBTITLE_NUMBER:
			detail = fileDetail(file) 
			+ "no subtitle number at line " + lineNumber;	
			break;
		case MALFORMED_SUBTITLE_NUMBER:
			detail = fileDetail(file) 
			+ "no subtitle number at line " + lineNumber 
			+ Utils.LINE_SEPARATOR + line;
			break;
		case NO_SUBTITLE_TIME:
			detail = fileDetail(file) 
			+ "no time at line " + lineNumber;
			break;
		case MALFORMED_TIME_LINE:
			detail = fileDetail(file) 
			+ "malformed time at line " + lineNumber
			+ Utils.LINE_SEPARATOR + line;		
			break;
		case MALFORMED_START_TIME:
			detail = fileDetail(file) 
			+ "malformed start time at line " + lineNumber 
			+ Utils.LINE_SEPARATOR + line;		
			break;
		case MALFORMED_END_TIME:
			detail = fileDetail(file) 
			+ "malformed end time at line " + lineNumber 
			+ Utils.LINE_SEPARATOR + line;
			break;
		case NO_SUBTITLE_TEXT:
			detail = fileDetail(file) 
			+ "no subtitle text at line " + lineNumber;
			break;
		case MALFORMED_SUBTITLE_TEXT:
			detail = fileDetail(file) 
			+ "no subtitle text at line " + lineNumber;
			break;
		case MALFORMED_SUBTITLE_FILE:
			detail = fileDetail(file) 
			+ "unknow error " + Utils.LINE_SEPARATOR
			+ "malformed subtitle file at line " + lineNumber;
			break;
		default:
			throw new IllegalArgumentException(type + " is not a valid identifier for an exception type");
		}
		return new Bean(message, detail, true);
	}

	/**
	 * Get a new <tt>FileConversionException</tt> instance
	 * @param file			the file in cause
	 * @param extension		the <tt>file</tt> extension
	 * @return				a unsupported file format exception
	 */
	public static FileConversionException getUnsupportedFileFormatException(File file, String extension) {
		return new Bean(
				// MESSAGE
				"unsupported file format",
				// DETAIL
				fileDetail(file) 
				+ extension + " file is not supported by Sears",
				// COULD BE THE RESULT OF A BAD ENCODING
				false
		);
	}
}

/**
 * This class provides a way to create a basic FileConversion object
 * <br> Used by the <tt>static</tt> methods of the abstract class that <tt>Bean</tt> extends
 */
class Bean extends FileConversionException {

	private static final long serialVersionUID = 75866932870197914L;

	private String message;
	private String detail;
	private Boolean couldBeABadEncoding;

	/**
	 * Constructs a new object with an error message,
	 * detail for this error and a <tt>boolean</tt>
	 * to know if the error could be occur because a bad encoding/decoding
	 * @param message				the error message
	 * @param detail				the detail of the error
	 * @param couldBeABadEncoding	if the error may cause by a bad decoding/encoding value is true, false if not
	 */
	public Bean(String message, String detail, boolean couldBeABadEncoding) {
		this.message = message;
		this.detail = detail;
		this.couldBeABadEncoding = couldBeABadEncoding;
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.exception.io.FileConversionException#couldBeABadEncoding()
	 */
	public boolean couldBeABadEncoding() {
		return couldBeABadEncoding;
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.exception.io.FileConversionException#getDetail()
	 */
	public String getDetail() {
		return detail;
	}

	/*
	 * (non-Javadoc)
	 * @see sears.file.exception.io.FileConversionException#getMessage()
	 */
	public String getMessage() {
		return message;
	}
}