/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

//some suggestions about this class: floriaen@gmail.com

package sears.gui;

import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.nio.charset.Charset;
import java.util.Iterator;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingConstants;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import sears.file.SubtitleFile;
import sears.gui.resources.SearsResources;
import sears.tools.Utils;

/**
 * <p>
 * <tt>ErrorDialog</tt> allows to display 3 things:
 * <ul>
 * <li>A title to display a general message to alert user</li>
 * <li>A message error which describes the error</li>
 * <li>A text area where is printed error's details</li>
 * <li>A panel with a combo box to let user choose another encoding</li>
 * </ul>
 * </p>
 * <p>
 * <tt>ErrorDialog</tt> provides static method for access to generics error dialog:
 * <br><tt># showOpenErrorDialog(String, String, boolean)</tt>
 * <br><tt># showSaveErrorDialog(String, String)</tt>
 * 
 */
public class ErrorDialog extends SearsJDialog {
	private static final long serialVersionUID = -8640938815129486410L;

	/** this font is used for all the dialog's components */
	public static final Font DEFAULT_FONT = UIManager.getFont("Label.font");
	/** derived font of <tt>DEFAULT_FONT</tt> */
	public static final Font BIG_FONT = DEFAULT_FONT.deriveFont(13f);
	/** derived font of <tt>DEFAULT_FONT</tt> */
	public static final Font MIDDLE_FONT = DEFAULT_FONT.deriveFont(12f);
	/** derived font of <tt>DEFAULT_FONT</tt> */
	public static final Font SMALL_FONT = DEFAULT_FONT.deriveFont(11f);

	private static final int MINIMUM_HEIGHT = 180;
	private static final int MINIMUM_WIDTH = 300;

	// MESSAGE PANEL
	private JPanel messagePanel;
	private JLabel mainTitle;
	private JLabel errorMessageLabel;
	private ArrowButton arrowButton;

	// DETAIL AREA
	private JPanel detailPanel;
	private JScrollPane scrollDetailPane;
	private JTextArea errorDetailArea;

	// CHARSET PANEL
	private JPanel charsetPanel;
	private JTextArea charsetMessageArea;
	private JComboBox charsetComboBox;

	// an instance used by the static method
	private static ErrorDialog openErrorDialog;

	/**
	 * Constructs a new instance of <tt>ErrorDialog</tt>
	 * <br>A preferred size is set as default
	 */
	public ErrorDialog() {
		super("Error");
		//this.setPreferredSize(new Dimension(200, 300));
	}

	/**
	 * Redefines super method, add components to the dialog before
	 * compute the it size :D
	 */
	protected void configureSize() {		
		// DEVELOPER:
		// the super method #configureSize() calls the pack() method if there's no saved sizes.
		// The problem resides in the fact that there's not all the components added when this method 
		// is called, so pack() method result is not good for the view...
		//
		//this.setMinimumSize(new Dimension(372,332));
		initComponents();
		setSize(MINIMUM_WIDTH, MINIMUM_HEIGHT);
		setLocationRelativeTo(MainWindow.instance);
		//super.configureSize();
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.Component#setVisible(boolean)
	 */
	public void setVisible(boolean aFlag) {
		if( aFlag == true ) {
			setSize(MINIMUM_WIDTH, MINIMUM_HEIGHT);
		}
		super.setVisible(aFlag);
	}

	/**
	 * Initializes all the dialog's components
	 */
	private void initComponents() {
		Container contentPane = getContentPane();
		contentPane.setLayout(new BorderLayout());
		contentPane.add(getMessagePanel(), BorderLayout.NORTH);
		contentPane.add(getDetailPanel(), BorderLayout.CENTER);
		contentPane.add(getCharsetPanel(), BorderLayout.SOUTH);
	}

	/**
	 * Constructs if needed and returns the message panel.
	 * <br> This panel contains two <tt>JLabel</tt> object:
	 * <ul>
	 * <li>one to describe to the user what happens</li>
	 * <li>the other to display which error occurs</li>
	 * </ul>
	 * @return the message panel
	 */
	private JPanel getMessagePanel() {
		if( messagePanel == null ) {
			messagePanel = new JPanel();
			messagePanel.setLayout(new GridLayout(2,0));
			messagePanel.setBorder(BorderFactory.createEmptyBorder(10, 4, 6, 4));
			// ERROR MESSAGE
			errorMessageLabel = new JLabel();
			errorMessageLabel.setFont(MIDDLE_FONT);
			errorMessageLabel.setHorizontalTextPosition(SwingConstants.LEFT);
			// MAIN TITLE
			mainTitle = new JLabel();
			Icon errorIcon = SearsResources.getIcon("ErrorIcon");
			mainTitle.setIconTextGap(5);
			mainTitle.setIcon(errorIcon);
			mainTitle.setFont(BIG_FONT);
			// ADD LABELS TO PANEL
			messagePanel.add(mainTitle);
			//messagePanel.add(errorMessageLabel);
			messagePanel.add(getArrowButton());
		}
		return messagePanel;
	}

	/**
	 * Constructs if needed and returns the "detail panel"
	 * <br>It is composed by an area where is displayed text
	 * @return	the panel
	 */
	private JPanel getDetailPanel() {
		if( detailPanel == null ) {
			detailPanel = new JPanel();
			detailPanel.setBorder(BorderFactory.createEmptyBorder(4, 4, 4, 4));
			scrollDetailPane = new JScrollPane(getErrorDetailArea());
			scrollDetailPane.setPreferredSize(new Dimension(200,100));
			detailPanel.setLayout(new GridLayout(1,0));
			detailPanel.add(scrollDetailPane);
		}
		return detailPanel;
	}

	// get the arrow button
	private JButton getArrowButton() {
		arrowButton = new ArrowButton("details");
		arrowButton.setHorizontalAlignment(SwingConstants.LEFT);
		arrowButton.setFont(MIDDLE_FONT);
		arrowButton.setToolTipText("click to show or hide error details");
		arrowButton.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				setDetailPanelVisible(!arrowButton.isTurnOn());
			}			
		});
		return arrowButton;
	}

	/**
	 * Used by the arrow button to show or hide the detail panel
	 * @param visible true for show, false for hide
	 */
	protected void setDetailPanelVisible(boolean visible) {
		if( visible && !detailPanel.isVisible() ) {
			detailPanel.setVisible(true);
			int detailPanelHeigth = detailPanel.getHeight();
			if( detailPanelHeigth == 0 ) {
				detailPanelHeigth = 100;
			}		
			this.setSize(this.getWidth(), this.getHeight() + detailPanelHeigth);
			this.repaint();
		} else if( !visible && detailPanel.isVisible() ) {
			detailPanel.setVisible(false);
			this.setSize(this.getWidth(), this.getHeight() - detailPanel.getHeight());
			this.repaint();
		}
	}

	/**
	 * Constructs if needed and returns a non-editable <tt>JTextArea</tt> object.
	 * <br>Use for display error messages
	 * @return	the error message area
	 */
	private JTextArea getErrorDetailArea() {
		if( errorDetailArea == null ) {
			errorDetailArea = new JTextArea();
			errorDetailArea.setEditable(false);
			errorDetailArea.setBorder(BorderFactory.createEmptyBorder(2,4,2,4));
			errorDetailArea.setFont(SMALL_FONT);
		}
		return errorDetailArea;
	}

	/**
	 * Constructs if needed and returns the "charset panel" composed by labels and a combo box
	 * <br>to give tu user the possibility to choose another charset encoding/decoding
	 * @return	the panel
	 */
	private JPanel getCharsetPanel() {
		if( charsetPanel == null ) {
			charsetPanel = new JPanel();
			charsetPanel.setBorder(BorderFactory.createEmptyBorder(6, 4, 6, 18));

			charsetMessageArea = new JTextArea();
			charsetMessageArea.setText(
					"A wrong charset might be the error origin."	
					+ Utils.LINE_SEPARATOR
					+ "You can try to reopen the file with the"
					+ Utils.LINE_SEPARATOR
					+ "charset of your choice:");

			charsetMessageArea.setFont(SMALL_FONT);
			charsetMessageArea.setEditable(false);
			charsetMessageArea.setBorder(BorderFactory.createEmptyBorder(2,4,2,4));
			charsetMessageArea.setBackground(charsetPanel.getBackground());

			charsetComboBox = new JComboBox();
			charsetComboBox.setFont(MIDDLE_FONT);
			fillCharsetComboBox();		
			charsetComboBox.addActionListener(new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					String selectedCharset = (String) charsetComboBox.getSelectedItem();
					charset = selectedCharset;
					// closed and dispose window dialog,
					// call to super method from SearsJDialog
					okAction();
				}			
			});

			charsetMessageArea.setAlignmentX(LEFT_ALIGNMENT);
			charsetComboBox.setAlignmentX(LEFT_ALIGNMENT);

			// adding components:
			charsetPanel.setLayout(new BoxLayout(charsetPanel, BoxLayout.Y_AXIS));
			charsetPanel.add(charsetMessageArea);
			charsetPanel.add(charsetComboBox);
		}
		return charsetPanel;
	}

	// NullPointerException if charsetComboBox is null
	private void fillCharsetComboBox() {
		String[] searsCharset = SubtitleFile.BASIC_CHARSETS;
		Iterator<Charset> availableCharsetsIt = Charset.availableCharsets().values().iterator();
		while( availableCharsetsIt.hasNext() ) {
			String charset = availableCharsetsIt.next().displayName();
			//charsetComboBox.addItem(charset);
			for( int i=0;i<searsCharset.length;i++ ) {
				if( charset.equals(searsCharset[i])) {
					charset = null;
					// exit the loop for:
					i = searsCharset.length;
				}
			}
			if( charset != null ) {
				charsetComboBox.addItem(charset);
			}
		}		
	}

	/**
	 * Indicates if the "charset panel" must be visible or not
	 * @param aFlag	if true, "charset panel" will be visible.
	 */
	public void setCharsetSelectionVisible(boolean aFlag) {
		charsetPanel.setVisible(aFlag);		
	}

	/**
	 * Sets the main title 
	 * @param title	the main title to set
	 */
	public void setMainTitle(String title) {
		mainTitle.setText(title);
	}

	/**
	 * Sets a message error
	 * @param message	the message error to set
	 */
	public void setErrorMessage(String message) {
		//errorMessageLabel.setText("> " + message);
		arrowButton.setText(message);
	}

	/**
	 * Sets a detail error
	 * @param detail	the detail error to set
	 */
	public void setErrorDetail(String detail) {
		// use line separator for a good display of the JScollBar
		errorDetailArea.setText(detail + Utils.LINE_SEPARATOR);
	}

	/**
	 * Appends text to the actual message error
	 * @param detail	the text to append
	 */
	public void appendErrorDetail(String detail) {
		String textArea = errorDetailArea.getText();
		errorDetailArea.setText(textArea + detail);
	}

	/**
	 * Clears the error detail
	 */
	public void clearErrorDetail() {
		errorDetailArea.setText(null);
	}

	/*
	 * (non-Javadoc)
	 * @see sears.gui.SearsJDialog#getDialogName()
	 */
	protected String getDialogName() {
		return "error";
	}

	public void windowClosed(WindowEvent e) {
		//super.windowClosed(e);
		arrowButton.turnOff();
	}

	//
	// STATIC FACTORY METHODS:
	//

	// used to return result of user action on error dialog
	private static String charset = SubtitleFile.DEFAULT_CHARSET;

	/**
	 * Shows a generic error dialog when file is opened
	 * @param message		the message
	 * @param detail		the error detail
	 * @param reOpenChoice	if there's a way to resolve the error with a different charset
	 * @return				the charset choosed by the user or null if not
	 */
	public static String showOpenErrorDialog(String message, String detail, boolean reOpenChoice) {
		if( openErrorDialog == null ) {
			openErrorDialog = new ErrorDialog();
		}
		openErrorDialog.setMainTitle("Sears could'nt open file");
		openErrorDialog.setErrorMessage(message);
		openErrorDialog.setErrorDetail(detail);
		openErrorDialog.setCharsetSelectionVisible(reOpenChoice);
		openErrorDialog.setDetailPanelVisible(false);

		if( SwingUtilities.isEventDispatchThread() ) {
			openErrorDialog.addWindowListener(new WindowAdapter() {
				public void windowClosing(WindowEvent e) {
					charset = null;
				}
			});
			openErrorDialog.setVisible(true);
		} else {
			try {
				SwingUtilities.invokeAndWait(new Runnable() {
					public void run() {
						openErrorDialog.addWindowListener(new WindowAdapter() {
							public void windowClosing(WindowEvent e) {
								charset = null;
							}
						});
						openErrorDialog.setVisible(true);
					}
				});
			} catch (java.lang.reflect.InvocationTargetException e) {
				e.printStackTrace();
			} catch (java.lang.InterruptedException e) {
				e.printStackTrace();
			}
		}
		return charset;	
	}

	/**
	 * Shows a generic error dialog when file is saved
	 * @param message		the message
	 * @param detail		the error detail
	 */
	public static void showSaveErrorDialog(String message, String detail) {
		if( openErrorDialog == null ) {
			openErrorDialog = new ErrorDialog();
		}

		openErrorDialog.setMainTitle("Sears could'nt save the file");
		openErrorDialog.setErrorMessage(message);
		openErrorDialog.setErrorDetail(detail);
		openErrorDialog.setCharsetSelectionVisible(false);
		openErrorDialog.setDetailPanelVisible(false);

		if( SwingUtilities.isEventDispatchThread() ) {
			openErrorDialog.setVisible(true);
		} else {
			try {
				SwingUtilities.invokeAndWait(new Runnable() {
					public void run() {
						openErrorDialog.setVisible(true);
					}
				});
			} catch (java.lang.reflect.InvocationTargetException e) {
				e.printStackTrace();
			} catch (java.lang.InterruptedException e) {
				e.printStackTrace();
			}
		}
	}
}