/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
package sears.gui;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

import sears.file.Subtitle;
import sears.file.SubtitleFile;
import sears.gui.resources.SearsResources;
import sears.tools.SearsResourceBundle;
import sears.tools.Trace;

/**
 * Class JDialogSplit.
 * <br><b>Summary:</b><br>
 * This Class permit to enter Split parameters.
 */
public class JDialogSplit extends SearsJDialog implements ChangeListener {
    private static final long serialVersionUID = -1582903664124772065L;

    private JPanel jContentPane = null;

    private JPanel jPanelCenter = null;

    private JPanel jPanelOutputFiles = null;

    private JLabel jLabelPart1 = null;

    private JTextField jTextFieldPart1 = null;

    private JButton jButtonPart1 = null;

    private JLabel jLabelPart2 = null;

    private JTextField jTextFieldPart2 = null;

    private JButton jButtonPart2 = null;

    private JPanel jPanelSubtitle = null;

    private JLabel jLabelSubtitle = null;

    private JSlider jSliderSubtitle = null;

    private JPanel jPanelChecks = null;

    private JCheckBox jCheckBoxDelay = null;

    private JTextField jTextFieldDelay = null;

    private JCheckBox jCheckBoxOpenSecond = null;

    /**The entered destination files. This array is filled in the checkParameters method*/
    private File[] destinationFiles;

    /**The subtitleList, that contains all the subtitles.*/
    private ArrayList<Subtitle> subtitleList;

    /**The first choosen subtitle and the last choosen one, when user validates the dialog.*/
    private int choosenSubtitle;

    /**The opened subtitleFile.*/
    private SubtitleFile subtitleFile;

    /**
     * This is the default constructor
     * @param _subtitleList 	the subtitle list
     * @param _choosenSubtitle 	the subtitle number where the split will occur
     * @param _subtitleFile 	the subtitle file
     */
    public JDialogSplit(ArrayList<Subtitle> _subtitleList, int _choosenSubtitle, SubtitleFile _subtitleFile) {
        super(SearsResourceBundle.getResource("split_title"));
        //Initialize attributes.
        choosenSubtitle = _choosenSubtitle;
        subtitleList = _subtitleList;
        validationStatus = false;
        subtitleFile = _subtitleFile;
        initializeDestinationFiles();
        setContentPane(getJContentPane());
        configureSize();
    }

    /**
     * Method initializeDestinationFiles.
     * <br><b>Summary:</b><br>
     * Call to method to initialize the destination files array.
     */
    private void initializeDestinationFiles() {
        //construct array.
        destinationFiles = new File[2];
        //Construct filename 1 (which will contains the first range)
        String fileName = subtitleFile.getFile().getName();
        //remove extension
        fileName = fileName.substring(0, fileName.lastIndexOf("."));
        destinationFiles[0] = new File(subtitleFile.getFile().getParent() + 
        		File.separator + fileName + "_1." + subtitleFile.extension());
        //construct filename 2
        destinationFiles[1] = new File(subtitleFile.getFile().getParent() + 
        		File.separator + fileName + "_2." + subtitleFile.extension());
    }

    /**
     * This method initializes jContentPane
     * 
     * @return javax.swing.JPanel
     */
    private JPanel getJContentPane() {
        if (jContentPane == null) {
            jContentPane = new JPanel();
            jContentPane.setLayout(new BorderLayout());
            jContentPane.setBorder(super.createEmptyBorder());
            jContentPane.add(getJPanelButtons(), java.awt.BorderLayout.SOUTH);
            jContentPane.add(getJPanelCenter(), java.awt.BorderLayout.CENTER);
        }
        return jContentPane;
    }

    /**
     * Method okAction.
     * <br><b>Summary:</b><br>
     * This method is called when user validate the dialog.
     */
    protected void okAction() {
        //Just have to check parameters validity, if valids, dispose dialog,and set validation status to true.
        String error = checkParameters();
        if (error != null && !error.equals("")) {
            //Show error message.
            JOptionPane.showMessageDialog(this, error, SearsResourceBundle.getResource("error_splitConfigurationError"),
                    JOptionPane.ERROR_MESSAGE);
            Trace.trace("Error in Split parameters:" + error, Trace.WARNING_PRIORITY);
        } else {
            //else split configuration is valid, release dialog.
            validationStatus = true;
            dispose();
        }
    }

    /**
     * Method checkParameters.
     * <br><b>Summary:</b><br>
     * This method permits to check the split parameters validity.
     * It sets the final parameters that will be returned by the public methods.
     * @return  <b>String</b>   "" if there is no error in parameters, error message otherwise.
     */
    private String checkParameters() {
        //The error message that will be returned.
        String errorMessage = "";
        //Check first destination file.
        errorMessage += checkFile(getJTextFieldPart1().getText(), 0);
        //check second file.
        errorMessage += checkFile(getJTextFieldPart2().getText(), 1);
        //check choosen subtitle.
        //nothing to check... slider is obsviously between min and max :/
        choosenSubtitle = getJSliderSubtitle().getValue();
        //check delay.
        if (getJCheckBoxDelay().isSelected()) {
            String delay = getJTextFieldDelay().getText();
            if (delay != null && !delay.equals("")) {
                try {
                    Double.parseDouble(delay);
                } catch (NumberFormatException e) {
                    //Delay value is not a number.
                    errorMessage += SearsResourceBundle.getResource("error_delayNotValid") + "\n";
                }
            } else {
                //Delay value is null.
                errorMessage += SearsResourceBundle.getResource("error_delayNull") + "\n";
            }
        }
        return errorMessage;
    }

    /**
     * Method checkFile.
     * <br><b>Summary:</b><br>
     * This method check a destination fileName validity.
     * @param fileName          The fileNAme to check.
     * @param destinationIndex  The destination index, 0 for first part, and 1 for the second part.
     * @return String           The new error message.
     */
    private String checkFile(String fileName, int destinationIndex) {
        String errorMessage = "";
        if (fileName != null && !fileName.equals("")) {
            File file = new File(fileName);
            try {
                if ((file.exists() && file.canWrite()) || file.createNewFile()) {
                    //set destination file
                    destinationFiles[destinationIndex] = file;
                } else {
                    //cannot write to file!
                    errorMessage += SearsResourceBundle.getResource("error_cannotWrite") + fileName + ".\n";
                }
            } catch (IOException e) {
                //cannot write to file!
                errorMessage += SearsResourceBundle.getResource("error_cannotWrite") + fileName + ".\n";
            }
        } else {
            //cannot write to file!
            errorMessage += SearsResourceBundle.getResource("error_destinationNotDefined1") + " " + (destinationIndex + 1) + " "
                    + SearsResourceBundle.getResource("error_destinationNotDefined2") + "\n";
        }
        return errorMessage;
    }

    /**
     * This method initializes jPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getJPanelCenter() {
        if (jPanelCenter == null) {
            GridBagConstraints gridBagConstraints12 = new GridBagConstraints();
            gridBagConstraints12.gridx = 0;
            gridBagConstraints12.gridy = 4;
            gridBagConstraints12.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints12.fill = java.awt.GridBagConstraints.HORIZONTAL;
            GridBagConstraints gridBagConstraints11 = new GridBagConstraints();
            gridBagConstraints11.fill = java.awt.GridBagConstraints.HORIZONTAL;
            gridBagConstraints11.gridx = 0;
            gridBagConstraints11.gridy = 2;
            gridBagConstraints11.gridheight = 2;
            GridBagConstraints gridBagConstraints = new GridBagConstraints();
            gridBagConstraints.gridx = 0;
            gridBagConstraints.gridheight = 2;
            gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
            gridBagConstraints.gridy = 0;
            jPanelCenter = new JPanel();
            jPanelCenter.setLayout(new GridBagLayout());
            jPanelCenter.add(getJPanelOutputFiles(), gridBagConstraints);
            jPanelCenter.add(getJPanelSubtitle(), gridBagConstraints11);
            jPanelCenter.add(getJPanelChecks(), gridBagConstraints12);
        }
        return jPanelCenter;
    }

    /**
     * This method initializes jPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getJPanelOutputFiles() {
        if (jPanelOutputFiles == null) {
            GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
            gridBagConstraints4.fill = java.awt.GridBagConstraints.HORIZONTAL;
            gridBagConstraints4.weightx = 1.0;
            gridBagConstraints4.gridx = 1;
            gridBagConstraints4.gridy = 1;
            GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
            gridBagConstraints5.gridx = 2;
            gridBagConstraints5.insets = new java.awt.Insets(1, 0, 7, 0);
            gridBagConstraints5.ipadx = 0;
            gridBagConstraints5.gridy = 1;
            GridBagConstraints gridBagConstraints3 = new GridBagConstraints();
            gridBagConstraints3.gridx = 0;
            gridBagConstraints3.gridy = 1;
            gridBagConstraints3.ipadx = 4;
            jLabelPart2 = new JLabel();
            jLabelPart2.setText(SearsResourceBundle.getResource("split_part") + " 2");
            jLabelPart1 = new JLabel();
            jLabelPart1.setText(SearsResourceBundle.getResource("split_part") + " 1");
            GridBagConstraints gridBagConstraints2 = new GridBagConstraints();
            gridBagConstraints2.gridx = 2;
            gridBagConstraints2.gridy = 0;
            GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
            gridBagConstraints1.gridx = 1;
            gridBagConstraints1.gridy = 0;
            gridBagConstraints1.fill = java.awt.GridBagConstraints.HORIZONTAL;
            gridBagConstraints1.weightx = 1.0;
            // jLabelPart1 constraints:
            GridBagConstraints gridBagConstraints0 = new GridBagConstraints();
            gridBagConstraints0.ipadx = 4;
            jPanelOutputFiles = new JPanel();
            jPanelOutputFiles.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), SearsResourceBundle
                    .getResource("split_outputFiles")));
            jPanelOutputFiles.setSize(400, 200);
            jPanelOutputFiles.setLayout(new GridBagLayout());
            jPanelOutputFiles.add(jLabelPart1, gridBagConstraints0);
            jPanelOutputFiles.add(getJTextFieldPart1(), gridBagConstraints1);
            jPanelOutputFiles.add(getJButtonPart1(), gridBagConstraints2);
            jPanelOutputFiles.add(jLabelPart2, gridBagConstraints3);
            jPanelOutputFiles.add(getJTextFieldPart2(), gridBagConstraints4);
            jPanelOutputFiles.add(getJButtonPart2(), gridBagConstraints5);
        }
        return jPanelOutputFiles;
    }

    /**
     * This method initializes jTextField	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getJTextFieldPart1() {
        if (jTextFieldPart1 == null) {
            jTextFieldPart1 = new JTextField(18);
            jTextFieldPart1.setText(destinationFiles[0].getAbsolutePath());
        }
        return jTextFieldPart1;
    }

    /**
     * This method initializes jButton	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getJButtonPart1() {
        if (jButtonPart1 == null) {
            jButtonPart1 = new JButton(SearsResources.getIcon("BrowseIcon"));
            jButtonPart1.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    browseDestinationFile(0);
                }
            });
        }
        return jButtonPart1;
    }

    /**
     * Method browseDestinationFile.
     * <br><b>Summary:</b><br>
     * Use thid method to choose destination files.
     * @param i     An <b>int</b> equals to 0 to choose first part, or 1 to choose second part.
     */
    protected void browseDestinationFile(int i) {
        File choosenFile = MainWindow.instance.showSRTBrowser();
        if (choosenFile != null) {
            if (i == 0) {
                getJTextFieldPart1().setText(choosenFile.getAbsolutePath());
            } else {
                getJTextFieldPart2().setText(choosenFile.getAbsolutePath());
            }
        }
    }

    /**
     * This method initializes jTextField	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getJTextFieldPart2() {
        if (jTextFieldPart2 == null) {
            jTextFieldPart2 = new JTextField(18);
            jTextFieldPart2.setText(destinationFiles[1].getAbsolutePath());
        }
        return jTextFieldPart2;
    }

    /**
     * This method initializes jButton	
     * 	
     * @return javax.swing.JButton	
     */
    private JButton getJButtonPart2() {
        if (jButtonPart2 == null) {
            jButtonPart2 = new JButton(SearsResources.getIcon("BrowseIcon"));
            jButtonPart2.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    browseDestinationFile(1);
                }
            });
        }
        return jButtonPart2;
    }

    /**
     * This method initializes jPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getJPanelSubtitle() {
        if (jPanelSubtitle == null) {
            jPanelSubtitle = new JPanel();
            jPanelSubtitle.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), SearsResourceBundle
                    .getResource("split_cutOnST")));
            jPanelSubtitle.setLayout(new GridLayout(2, 1));
            jPanelSubtitle.add(getJLabelSubtitle(), null);
            jPanelSubtitle.add(getJSliderSubtitle(), null);
        }
        return jPanelSubtitle;
    }

    /**
     * This method initialize the JLabelSubtitle.
     * 
     * @return javax.swing.JLabel
     */
    private JLabel getJLabelSubtitle() {
        if (jLabelSubtitle == null) {
            jLabelSubtitle = new JLabel();
            setSubtitleLabel(choosenSubtitle);
        }
        return jLabelSubtitle;
    }

    /**
     * This method initializes jSlider	
     * 	
     * @return javax.swing.JSlider	
     */
    private JSlider getJSliderSubtitle() {
        if (jSliderSubtitle == null) {
            jSliderSubtitle = new JSlider(0, subtitleList.size() - 1, choosenSubtitle);
            jSliderSubtitle.addChangeListener(this);
        }
        return jSliderSubtitle;
    }

    /**
     * This method initializes jPanel	
     * 	
     * @return javax.swing.JPanel	
     */
    private JPanel getJPanelChecks() {
        if (jPanelChecks == null) {
            GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
            gridBagConstraints8.gridx = 0;
            gridBagConstraints8.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints8.gridwidth = 2;
            gridBagConstraints8.gridy = 1;
            GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
            gridBagConstraints7.fill = java.awt.GridBagConstraints.NONE;
            gridBagConstraints7.weightx = 1.0;
            GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
            gridBagConstraints6.anchor = java.awt.GridBagConstraints.WEST;
            jPanelChecks = new JPanel();
            jPanelChecks.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), SearsResourceBundle
                    .getResource("split_options")));
            jPanelChecks.setLayout(new GridBagLayout());
            jPanelChecks.add(getJCheckBoxDelay(), gridBagConstraints6);
            jPanelChecks.add(getJTextFieldDelay(), gridBagConstraints7);
            jPanelChecks.add(getJCheckBoxOpenSecond(), gridBagConstraints8);
        }
        return jPanelChecks;
    }

    /**
     * This method initializes jCheckBox	
     * 	
     * @return javax.swing.JCheckBox	
     */
    private JCheckBox getJCheckBoxDelay() {
        if (jCheckBoxDelay == null) {
            jCheckBoxDelay = new JCheckBox(SearsResourceBundle.getResource("split_delay2ndPart"));
        }
        return jCheckBoxDelay;
    }

    /**
     * This method initializes jTextField	
     * 	
     * @return javax.swing.JTextField	
     */
    private JTextField getJTextFieldDelay() {
        if (jTextFieldDelay == null) {
            jTextFieldDelay = new JTextField("1", 4);
            jTextFieldDelay.setToolTipText(SearsResourceBundle.getResource("delay_tip"));
        }
        return jTextFieldDelay;
    }

    /**
     * This method initializes jCheckBox	
     * 	
     * @return javax.swing.JCheckBox	
     */
    private JCheckBox getJCheckBoxOpenSecond() {
        if (jCheckBoxOpenSecond == null) {
            jCheckBoxOpenSecond = new JCheckBox(SearsResourceBundle.getResource("split_open2ndPart"));
        }
        return jCheckBoxOpenSecond;
    }

    /**
     * Method getDestinationsFiles.
     * <br><b>Summary:</b><br>
     * This method returns the entered destinations files.
     * @return  <b>Files[]</b>      The entered destination files. or null if user didn't validate.
     */
    public File[] getDestinationsFiles() {
        //The result of the method.
        return destinationFiles;
    }

    /* (non-Javadoc)
     * @see javax.swing.event.ChangeListener#stateChanged(javax.swing.event.ChangeEvent)
     */
    public void stateChanged(ChangeEvent e) {
        //When slider change of value, change the content of the jLabelSubtitle.
        if (e.getSource() == jSliderSubtitle) {
            setSubtitleLabel(jSliderSubtitle.getValue());
        }
    }

    /**
     * Method setSubtitleLabel.
     * <br><b>Summary:</b><br>
     * Call this method to fill the subtitle label with the content of the subtitle at the given index. 
     * @param index     The <b>int</b> index to find the subtitle.
     */
    private void setSubtitleLabel(int index) {
        //Get the subtitle at the given index.
        Subtitle subtitle = (Subtitle) subtitleList.get(index);
        //construct message to show in label.
        //Put ST number, and append the message.
        String message = "[" + subtitle.getNumber() + "]" + subtitle.getSubtitle();
        //Cut long ST, not to perturb GUI.
        int maxLength = 35;
        if (message.length() > maxLength) {
            message = message.substring(0, maxLength - 1) + "...";
        }
        //set the label to the content of the subtitle.
        getJLabelSubtitle().setText(message);
    }

    /**
     * Method getDelay.
     * <br><b>Summary:</b><br>
     * This method return the delay entered for the second part.
     * @return  double     The delay entered for the second part, or 1 if not delay is necessary (1s will be the delay by default).
     */
    public double getSecondPartDelay() {
        //the result of the method.
        double result = -1;
        if (getJCheckBoxDelay().isSelected()) {
            try {
                //get the delay from the textField.
                result = Double.parseDouble(getJTextFieldDelay().getText());
            } catch (NumberFormatException e) {
                Trace.trace("Bad number entered in Delay ! Delay is ignored.", Trace.ERROR_PRIORITY);
                //set a delay of 1
                result = 1;
            }
        }
        //return the result
        return result;
    }

    /**
     * Method needToOpenSecondPart.
     * <br><b>Summary:</b><br>
     * Return true if the open second part checkBox is selected.
     * @return  boolean True if the open second part checkBox is selected. False otherwise.
     */
    public boolean needToOpenSecondPart() {
        //return true if checkBox is selectd.
        return getJCheckBoxOpenSecond().isSelected();
    }

    /**
     * Method getChoosenSubtitle.
     * <br><b>Summary:</b><br>
     * This method return the choosen subtitle index for spliting.
     * @return  <b>int</b>  The choosen subtitle index for spliting.
     */
    public int getChoosenSubtitle() {
        return choosenSubtitle;
    }

    /* (non-Javadoc)
     * @see sears.gui.SearsJDialog#getDialogName()
     */
    protected String getDialogName() {
        return "split";
    }
}
