/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

package sears.gui;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.util.ArrayList;

import javax.swing.BorderFactory;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.UIManager;
import javax.swing.table.DefaultTableCellRenderer;

import sears.file.Subtitle;
import sears.file.SubtitleFile;
import sears.gui.resources.SearsResources;
import sears.tools.SearsResourceBundle;

/**
 * Class SubtitleTableCellRenderer This renderer permits to display the anchor
 * in the 4rth cell. It also permits to change the color of the row's cell if
 * the subtitle is anchored.
 */
public class SubtitleTableCellRenderer extends DefaultTableCellRenderer{

	/** (<b>long</b>) serialVersionUID: The serialVersionUID */
	private static final long serialVersionUID = 6305494600328229080L;

	/** (<b>ImageIcon</b>) anchorIcon: The anchorIcon */
	private static ImageIcon anchorIcon;

	/** (<b>ImageIcon</b>) dotIcon: The anchorIcon */
	private static ImageIcon dotIcon;

	/** (<b>Color</b>) oldBackgroundColor: The oldBackgroundColor */
	private static Color oldBackgroundColor;

	/** (<b>Color</b>) oldSelectedBackgroundColor: The oldSelectedBackgroundColor */
	private static Color oldSelectedBackgroundColor;

	/** (<b>Color</b>) ANCHORED_BACKGROUND_COLOR: The color to be used when ST is anchored */
	private static final Color ANCHORED_BACKGROUND_COLOR = new Color(190,240,190);
	
	/** the default font used by component */
	public static final Font DEFAULT_FONT = UIManager.getFont("Label.font");
	
	/** the color of the background component when it is selected */
	public static final Color DEFAULT_SELECTION_BACKGROUND = UIManager.getColor("Table.selectionBackground");
	
	/** the color of the component background*/
	public static final Color DEFAULT_BACKGROUND = UIManager.getColor("Table.background");
	
	/** (<b>ArrayList<Subtitle></b>) subtitleList: The subtitleList. */
	private ArrayList<Subtitle> subtitleList;
	
	// the cell component:
	private JLabel cellHandle;	
	
	private String highlightedText;
	
	/**
	 * Constructor SubtitleTableCellRenderer. <br>
	 * <b>Summary:</b><br>
	 * The constructor of the class SubtitleTableCellRenderer
	 * 
	 * @param subtitleList
	 *            The subtitleList.
	 */
	public SubtitleTableCellRenderer(ArrayList<Subtitle> subtitleList) {		
		this.subtitleList = subtitleList;
		//save the background color
		oldBackgroundColor = DEFAULT_BACKGROUND;
		highlightedText = null;
	}

	/**
	 * a call to this method return the appropriate <code>String</code> instance
	 * for an <code>Object<code> object that could be a <code>String</code> or an <code>Integer</code>
	 * @param value 	the object to reveal
	 * @return			the same object as a <code>String</code> object or null, if the object does not correspond
	 */
	private String objectToString(Object value) {
		String str = null;
		if( value instanceof Integer ) {
			str = String.valueOf((Integer) value);
		} else if( value instanceof String ) {
			str = (String) value;
		}
		// else return a null string
		return str;
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.table.TableCellRenderer#getTableCellRendererComponent(javax.swing.JTable, java.lang.Object, boolean, boolean, int, int)
	 */
	public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int column) {
		super.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
	
		if( column == SubtitleTableModel.SUBTITLE_COLUMN ) {			
			SubtitleCellComponent cell = new SubtitleCellComponent(table.getRowHeight(row));			
			cell.setHighlightedString(highlightedText);
			cellHandle = cell;
		} else {
			cellHandle = new JLabel();
		}
		// set the visibility of effects:
		cellHandle.setOpaque(true);
		// homogeneity for all the cells:
		cellHandle.setFont(getFont());
		cellHandle.setBorder(BorderFactory.createEmptyBorder(0, 2, 0, 0));
		
		// set the text:
		String str = objectToString(value);
		if(str != null) {
			cellHandle.setText(str);
		}

		// display the selected row		
		if(isSelected) {
			cellHandle.setBackground(DEFAULT_SELECTION_BACKGROUND);
		} else {
			cellHandle.setBackground(null);
		}

		if(oldBackgroundColor == null && isSelected == false){
			oldBackgroundColor = cellHandle.getBackground();
		}
		if(oldSelectedBackgroundColor == null && isSelected == true){
			oldSelectedBackgroundColor = cellHandle.getBackground();
		}
		// And now, add an anchor if subtitle is anchored.
		Subtitle subtitle = subtitleList.get(row);
		if (subtitle.isAnchored()) {
			if (column == SubtitleTableModel.ANCHOR_COLUMN) {
				cellHandle.setIcon(getAnchorIcon());
				cellHandle.setHorizontalAlignment(JLabel.CENTER);
			}
			cellHandle.setToolTipText(SearsResourceBundle.getResource("anchor_anchoredTo")
					+ " " + SubtitleFile.timeToString(subtitle.getAnchor()));
			cellHandle.setBackground(ANCHORED_BACKGROUND_COLOR);
		} else {
			if (column == SubtitleTableModel.ANCHOR_COLUMN) {
				cellHandle.setIcon(getDotIcon());
				cellHandle.setHorizontalAlignment(JLabel.CENTER);
			}
			cellHandle.setToolTipText(null);
			if(isSelected){
				cellHandle.setBackground(oldSelectedBackgroundColor);
			} else{
				cellHandle.setBackground(oldBackgroundColor);
			}
		}
		// return the cell component:
		return cellHandle;
	}

	/**
	 * Method getAnchorIcon. <br>
	 * <b>Summary:</b><br>
	 * Return the anchor icon. Use a cache to increase performance since the
	 * anchor icon is always the same.
	 * 
	 * @return (<b>Icon</b>) The anchor icon.
	 */
	private Icon getAnchorIcon() {
		if (anchorIcon == null) {
			anchorIcon = SearsResources.getIcon("AnchorIcon");
		}
		return anchorIcon;
	}

	/**
	 * Method getDotIcon. <br>
	 * <b>Summary:</b><br>
	 * Return the dot icon. Use a cache to increase performance since the
	 * dot icon is always the same.
	 * 
	 * @return (<b>Icon</b>) The dot icon.
	 */
	private Icon getDotIcon() {
		if (dotIcon == null) {
			dotIcon = SearsResources.getIcon("DotIcon");
		}
		return dotIcon;
	}

	/**
	 * Call this method to highlight all occurrences of <tt>str</tt>
	 * in subtitle text view
	 * @param str	the string to highlight, if is null no view changes is done
	 */
	public void highlightString(String str) {
		highlightedText = str;
		
	}
}