/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

// some suggestions about this class: floriaen@gmail.com

package sears.gui.glassPane;

import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;

import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JViewport;
import javax.swing.SwingUtilities;

/**
 * <p>
 * This class provides a way to have a virtual rectangle over a viewport
 * with viewport size and position.
 * <br>Let the register listener and a resize of the viewport will affect the virtual bounds.
 * <br>The advantage of this resides in the fact that you can do the same things out and inside of 
 * this virtual bounds.
 * </p>
 * <b>How to use it:</b>
 * <pre>
 * <i>// init components:</i>
 * JComponent component = new JComponent();
 * JScrollPane scrollPane = new JScrollPane(component);
 * frame.add(scrollPane);
 * 	
 * <i>// set up the <code>ViewportGlassPane</code> object</i>
 * ViewportGlassPane gp = new ViewPortGlassPane(scrollPane.getViewport());
 * <i>// this method simply call frame.setGlassPane(gp) method</i>
 * gp.addAsAGlassPane(frame);
 * 
 * (...)
 * 	
 * EventQueue.invokeLater(new Runnable() {
 * 	public void run() {
 * 		gp.setVisible(true);
 * 	}
 * });
 * </pre>
 * 
 * @see JViewport
 */
public abstract class ViewportGlassPane extends JComponent {

	private static final long serialVersionUID = 5504986050347441090L;

	private JViewport viewport;	
	private Rectangle virtualBounds;
	private ViewportListener listener;

	/**
	 * Constructs a new <code>ViewportGlassPane</code> instance
	 * <br>Register a component listener, instance of <code>ViewportListener</code> to the viewport.
	 * <br>Resizing the view (viewport) will have no indesirable effect on the virtual bounds
	 * 
	 * @param viewport	the viewport on which the virtual bounds is set
	 * @throws 			NullPointerException if viewport is null
	 */
	public ViewportGlassPane(JViewport viewport) {
		super();
		if( viewport == null ) {
			throw new NullPointerException("viewport cannot be null");
		}
		this.viewport = viewport;
		listener = new ViewportListener(this);
		viewport.addComponentListener(listener);
	}
	
	/**
	 * Removes the listener associate to the viewport by the class constructor
	 * <br><b>SECURITY:</b>
	 * this method require that object which calls it knows and have an access 
	 * to the viewport gives in parameter of the class constructor
	 * 
	 * @param viewport 	must be equals to the one given in the class constructor parameter
	 * @see 			Object#equals(Object)
	 */
	protected void removeViewportListener(JViewport viewport) {
		if( viewport.equals(this.viewport)) {
			viewport.removeComponentListener(listener);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see javax.swing.JComponent#paint(java.awt.Graphics)
	 */
	public void paintComponent(Graphics g) {
		super.paintComponent(g);
		if( g instanceof Graphics2D ) {
			Graphics2D gr = (Graphics2D) g;
			paintComponentWithGraphics(gr);
		}
	}

	/**
	 * This method is called in <code>paint(g)</code> method.
	 * Allow a way to use the graphics context of this component
	 * 
	 * @param gr the graphics context of this component
	 */
	protected abstract void paintComponentWithGraphics(Graphics2D gr);

	/**
	 * When viewport changed when a resize is set on it.
	 * And so, is needed to recalculate the virtual bounds
	 */
	public void fireViewportSizeChanged() {
		virtualBounds = null;
	}

	/**
	 * Determines, if it is not already do, the virtual bounds size and position
	 * @return the virtual bonds as a <code>Rectangle</code> object
	 */
	public Rectangle getVirtualBounds() {
		if( virtualBounds == null ) {
			virtualBounds = SwingUtilities.convertRectangle(viewport, viewport.getVisibleRect(), this);
		}
		return virtualBounds;
	}
	
	/**
	 * Sets this as the glass pane of the frame given in parameters.
	 * If frame is null this method is a no op.
	 * @param frame	if null nothing is done
	 * @see JFrame#setGlassPane(java.awt.Component)
	 */
	public void addAsAGlassPane(JFrame frame) {
		if( frame != null ) {		
			frame.setGlassPane(this);
		}
	}

	/**
	 * Convert and return the component's graphics context
	 * @return 	the component's graphics context or null if it is null 
	 * 			or not instance of <code>Graphics2D</code>
	 */
	@Deprecated
	protected Graphics2D getGraphics2D() {
		Graphics2D gr = null;
		Graphics g = getGraphics();
		if( g != null ) {
			if( g instanceof Graphics2D ) {
				gr = (Graphics2D) g;
			}
		}
		return gr;
	}
}