/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

// some suggestions about this class: floriaen@gmail.com

package sears.search.data;

import java.util.ArrayList;

import sears.file.Subtitle;

/**
 * Data structure to manage result of a search.
 * <br><tt>CharIndexIterator</tt> performs a loop iteration
 *
 */
public class CharIndexIterator {
	
	private Window window;
	private LoopIterator loopIterator;
	private SubtitleFile subtitleFile;

	ArrayList<Subtitle> subtitleList;
	private String textForTheSearch;

	/**
	 * Constructs a new <tt>CharIndexIterator</tt> object
	 * @param subtitleList	the subtitle list
	 * @param str			the searched string 
	 * @throws IllegalArgumentException if one of the arguments id <tt>null</tt>
	 */
	public CharIndexIterator(ArrayList<Subtitle> subtitleList, String str) {
		if( subtitleList == null || subtitleList.isEmpty() ) {
			throw new IllegalArgumentException("the list of subtitles is null or empty");
		}
		
		window = new Window();
		this.subtitleList = subtitleList;
		subtitleFile = new SubtitleFile(subtitleList);
		setANewTextForTheSearch(str);
	}

	/**
	 * Sets a new array of <tt>Subtitle</tT> object
	 * @param newSubtitleList	the new array
	 */
	public void setANewSubtitleList(ArrayList<Subtitle> newSubtitleList) {
		subtitleFile = new SubtitleFile(newSubtitleList);
		loopIterator = subtitleFile.getAllRowWhichContainsAtLeastOneOccurrenceOfText(textForTheSearch).iterator();
		window = new Window();
	}

	/**
	 * Set if needed a new text for the search
	 * @param str 		the new text
	 * @return			true if the set is needed, false if not
	 */
	public boolean setANewTextForTheSearch(String str) {
		if( !isAValidText(str) ) {
			throw new IllegalArgumentException("the string is not valid");
		}
		str = isTextChanged(str);
		if( str != null) {
			setTextForTheSearch(str);
		}
		return (str != null);
	}

	private void setTextForTheSearch(String newText) {
		if( !isAValidText(newText) ) {
			throw new IllegalArgumentException("the string entered is not a valid string");
		}
		textForTheSearch = newText;
		fireTextChanged();
	}

	//private void resetWindow

	private void fireTextChanged() {
		loopIterator = subtitleFile.getAllRowWhichContainsAtLeastOneOccurrenceOfText(textForTheSearch).iterator();
		window = window.getEmptyClonedWindow();
	}

	/**
	 * Tests the <code>String</code> object given in parameter.
	 * @param textToTest the <code>String</code> object to test
	 * @return <code>textToTest</code> to lower case if it is different that the text stored, <code>null</code> if not
	 */
	private String isTextChanged(String textToTest) {
		textToTest = textToTest.toLowerCase();
		if( textForTheSearch != null && textForTheSearch.contentEquals(textToTest) ) {
			textToTest = null;
		}		
		return textToTest;
	}

	private boolean isAValidText(String text) {
		return ( text != null && text.trim().length() > 0 );
	}

	// ACCESS METHODS
	//

	/**
	 * Initializes and returns an array of two int
	 * <br>if <tt>row</tt> or <tt>index</tt> are negative, null is returned
	 * @param row	the row
	 * @param index	a char index
	 * @return 		an array of two int or null	
	 */
	private int[] rowAndCharIndex(int row, int index) {
		int[] rowAndCharIndex = new int[2];
		if( row < 0 || index < 0 ) {
			rowAndCharIndex = null;
		} else {
			rowAndCharIndex[0] = row;
			rowAndCharIndex[1] = index;
		}
		return rowAndCharIndex;
	}

	/**
	 * Returns the subtitle text at the index given in parameters
	 * @param index	the index in the list
	 * @return		a string or null if <tt>index</tt> is out of list bounds
	 */
	private String getSubtitleAtIndex(int index) {
		String subtitle = null;
		// ensure coherence with the subtitleList
		if( index >= 0 && index < subtitleList.size() ) {
			subtitle = subtitleList.get(index).getSubtitle();
		}
		return subtitle;
	}

	/**
	 * Gets the next row and char index
	 * <br> INCOHERENCE: row < 0 or row > last row...
	 * @return an array of two positive int or null if an incoherence appears during the method
	 * 
	 */
	public int[] getNextRowAndCharIndex() {
		int charIndex = window.getNextCharIndex();
		if( charIndex == -1 ) {
			int row = loopIterator.getNextElement();
			if( row != -1 ) {				
				String subtitle = getSubtitleAtIndex(row);
				// test the coherence
				if( subtitle != null ) {
					// update window:
					window.setWindowForGettingNextElement(subtitle, textForTheSearch);
					charIndex = window.getNextCharIndex();
					// test the coherence of the result:
					if( charIndex >= (subtitle.length() - textForTheSearch.length()) ) {
						// negative charIndex cause that the #rowAndCharMethod(int,int) method
						// returns null
						charIndex = -1;
					}
				} else {
					charIndex = -1;
				}
			}
		}
		return rowAndCharIndex(loopIterator.getCurrentElement(), charIndex);
	}

	/**
	 * Gets the row and first char index of the previous occurrence
	 * @return	an array of two <tt>int</tt>, the row and the first char index of the searched text
	 * 			null if there's no previous occurrence (this means that there's no occurrence at all
	 */
	public int[] getPreviousRowAndCharIndex() {
		int charIndex = window.getPreviousCharIndex();
		if( charIndex == -1 ) {
			int row = loopIterator.getPreviousElement();
			if( row != -1 ) {				
				String subtitle = getSubtitleAtIndex(row);
				// test the coherence
				if( subtitle != null ) {
					// update window:
					window.setWindowForGettingPreviousElement(subtitle, textForTheSearch);
					charIndex = window.getPreviousCharIndex();
					// test the coherence of the result:
					if( charIndex >= (subtitle.length() - textForTheSearch.length()) ) {
						// negative charIndex cause that the #rowAndCharMethod(int,int) method
						// returns null
						charIndex = -1;
					}
				} else {
					charIndex = -1;
				}
			}
		}
		return rowAndCharIndex(loopIterator.getCurrentElement(), charIndex);
	}

	/**
	 * Gets the row and first char index of the next occurrence after <tt>row</tt>
	 * @param row the row
	 * @return	an array of two <tt>int</tt>, the row and the first char index of the searched text
	 */
	public int[] getNextRowAndCharIndexBeginAtRow(int row) {
		int charIndex = -1;
		if( row >= 0 && row < subtitleList.size() ) {
			row = loopIterator.getTheNearestElementAfter(row);
			String subtitle = getSubtitleAtIndex(row);
			// ensures that row is valid compared to the subtitle list
			// row for table/index for subtitleList (same thing)
			if( subtitle != null ) {
				window.setWindowForGettingNextElement(subtitle, textForTheSearch);
				charIndex = window.getNextCharIndex();
				// test the coherence of the result:
				if( charIndex >= (subtitle.length() - textForTheSearch.length()) ) {
					// negative charIndex cause that the #rowAndCharMethod(int,int) method
					// returns null
					charIndex = -1;
				}
			}// else charIndex remain equals to -1, return value will be null 
		}
		return rowAndCharIndex(row, charIndex);
	}

	/**
	 * Gets the row and first char index of the previous occurrence after <tt>row</tt>
	 * @param row the row
	 * @return	an array of two <tt>int</tt>, the row and the first char index of the searched text
	 */
	public int[] getPreviousRowAndCharIndexBeginAtRow(int row) {
		int charIndex = -1;
		if( row >= 0 && row < subtitleList.size() ) {
			row = loopIterator.getTheNearestElementBefore(row);
			String subtitle = getSubtitleAtIndex(row);
			// ensures that row is valid compared to the subtitle list
			// row for table/index for subtitleList (same thing)
			if( subtitle != null ) {
				window.setWindowForGettingPreviousElement(subtitle, textForTheSearch);
				charIndex = window.getPreviousCharIndex();
				// test the coherence of the result:
				if( charIndex >= (subtitle.length() - textForTheSearch.length()) ) {
					// negative charIndex cause that the #rowAndCharMethod(int,int) method
					// returns null
					charIndex = -1;
				}
			}// else charIndex remain equals to -1, return value will be null 
		}
		return rowAndCharIndex(row, charIndex);
	}
}