/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

//some suggestions about this class: floriaen@gmail.com

package sears.search.gui;

import java.awt.Component;
import java.awt.Rectangle;

import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;

/**
 * Gives method to manipulate table view
 * [...]
 * <p>
 * Scroll to visible a table cell with a better style than
 * the {@link JTable#scrollRectToVisible(Rectangle)} method
 *</p>
 */
public class JTableManipulation {
	
	private static final int SIZE_DECAY = 20;

	private JTable table;
	private JScrollPane scrollPane;

	private int lastRow;
	private int lastColumn;

	/**
	 * Constructs a new <code>JTableManipulation</code> object
	 * @param scrollPane
	 * @throws NullPointerException
	 * @throws IllegalArgumentException
	 */
	public JTableManipulation(JScrollPane scrollPane) {
		if( scrollPane == null ) {
			throw new NullPointerException("JScrollPane object cannot be null");
		}
		// try to get back view
		Component view = scrollPane.getViewport().getView();
		if( view != null && view instanceof JTable ) {
			table = (JTable) view;
		} else {
			throw new IllegalArgumentException("the JScrollPane object is not associated to a JTable object or this object is null");
		}
		this.scrollPane = scrollPane;
		resetColumn();
	}

	/**
	 * Gets the previous row of <tt>row</tt>
	 * <br>Ensure loop coherence
	 * @param row	the row
	 * @return		the previous row
	 */
	private int getPreviousRow(int row) {
		if( --row < 0 ) {
			row = table.getRowCount();
		}
		return row;
	}
	
	/**
	 * Gets the next row of <tt>row</tt>
	 * <br>Ensure loop coherence
	 * @param row	the row
	 * @return		the next row
	 */
	private int getNextRow(int row) {
		if( ++row > table.getRowCount() ) {
			row = 0;
		}
		return row;
	}
	
	/**
	 * Reset <tt>lastColumn</tt>
	 */
	public void resetColumn() {
		lastColumn = -1;
	}
	
	/**
	 * Scroll to visible a table cell with a better style than
	 * the {@link JTable#scrollRectToVisible(Rectangle)} method.
	 * <br><tt>column</tt> must be coherent and must be the same 
	 * that the previous given in parameters of this method.
	 * <br>call {@link #resetColumn()} before ...
	 * @param row		the row to display in view
	 * @param column	the column
	 * @param backward 	the direction 
	 * @throws IllegalArgumentException if <tt>column</tt> is not the same than the last used <tt>column</tt>
	 */
	public void scrollCellToVisible(int row, int column, boolean backward) {
		if( lastColumn >= 0 && column != lastColumn ) {
			throw new IllegalArgumentException("Given column " + column + " must be equal to " + lastColumn);
		}
		if( row >= 0 && column >= 0 ) {
			int rowCount = table.getRowCount();
			if( row < rowCount && column < table.getColumnCount() ) {
				Rectangle cell = table.getCellRect(row, column, false);
				Rectangle convertedCell = SwingUtilities.convertRectangle(scrollPane.getViewport().getView(), cell, scrollPane.getViewport());
				Rectangle bounds = scrollPane.getBounds();
				// if previous or next cell is not on view
				if( !bounds.contains(convertedCell) ) {
					int size = cell.height;
					int viewSize = scrollPane.getViewport().getHeight();
					// condition to loop
					boolean loop = ( backward && row > lastRow ) || ( !backward && row < lastRow );
					while( !loop && size < viewSize - SIZE_DECAY ) {
						if( backward && ( row-- < getPreviousRow(row) ) ) {
							loop = true;
						} 
						if( !backward && ( row++ > getNextRow(row) ) ) {
							loop = true;
						}
						cell = table.getCellRect(row, column, true);
						size += cell.height;
					}
					// display cell in view
					table.scrollRectToVisible(cell);
				}
			}			
		}
		lastRow = row;
		lastColumn = column;
	}
}