/////////////////////////////////////////////////
//This file is part of Sears project.
//Subtitle Editor And Re-Synch
//A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is availbale under sourceforge
//at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
/////////////////////////////////////////////////

package sears.tools;

import java.awt.FileDialog;
import java.awt.Frame;
import java.io.File;
import java.io.FilenameFilter;
import java.util.Locale;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.UIManager;
import javax.swing.filechooser.FileFilter;


/**
 * This class gives methods for display an "open" dialog and a "save as" dialog.
 * <br>An instance of this class will display (if the good method is called),
 * <br>after test conditions a FileDialog or JFileChooser.
 * <br>
 * <br> <Strong>Example of use:</Strong>
 * <Blockquote>		DialogUtils du = new DialogUtils();
 * <br>				<i> // set options: </i>
 * <br>				du.setConditions( true, true );
 * <br>				du.setApproveButtonText( "Save as" );
 * <br>				du.setOverwriteText(SearsResourceBundle.getResource( "Do you really want to overwrite this file ?" ));
 * <br>				du.setFileFilter(new FileFilter() { ...	});	
 * <br>				<i>	// we get back the file choosen	</i>
 * <br>				File file = du.showSaveAsDialog(instance,  "Save As", subtitleFile.getFile());
 * <br>				<i>	// if user cancel action, file is null:	</i>
 * <br>				if (file != null){
 * <Blockquote> 		<i>	// action you want	</i>
 * </Blockquote>	}
 * </Blockquote>
 * 
 */
public class DialogUtils {
	//Conditions:

	// by default conditions are set to false, we display a JFileChooser
	private boolean isMacCondition = false;
	private boolean lookAndFeelCondition = false;
	
	// this condition should be necessary when you display FileDialog on Windows platform
	// method setFilenameFilter() doesn't work on this OS.
	private boolean isWindowsCondition = false;

	private String approveButtonText = null;

	private FileFilter filter = null;

	private String overwriteText = null;
	
	private Locale locale = null;

	public DialogUtils(){
		//...
	}

	/**
	 * <br>This method set conditions for display the FileDialog instead of the JFileChooser.
	 * @param OSCondition 			if true, the class instance will test if app is running on a Mac platform
	 * @param lookAndFeelCondition 	if true, the class instance will test if app's look and feel is the same of the system default one
	 */
	public void setConditions(boolean OSCondition , boolean lookAndFeelCondition){
		if(OSCondition){			
			// if we want to test if app is running on Mac, the windows condition is set to false:
			setConditions(false, OSCondition, lookAndFeelCondition);
		} else {
			// if we don't want to test if the app is running on a Mac platform,
			// we must test if the app is running on a Windows platform:
			setConditions(true, true, lookAndFeelCondition);
		}
	}

	private void setConditions(boolean windowsCondition, boolean macCondition, boolean lookAndFeelCondition){
		if(windowsCondition){
			this.isWindowsCondition = Utils.isWindowsPlatform;
		}		
		if(macCondition){
			// true if app is running on a Mac OS platform, false if not
			this.isMacCondition = Utils.isMacPlatform;
		}
		if(lookAndFeelCondition){
			// true if the app's look and feel is the same as the default system look and feel
			this.lookAndFeelCondition = (UIManager.getLookAndFeel()).toString().contains((UIManager.getSystemLookAndFeelClassName()));
		}
	}

	/**
	 * <br>this method set the approveButtonText needed by JFileChooser
	 * @param approveButtonText the text display in the approve button
	 */
	public void setApproveButtonText(String approveButtonText){
		this.approveButtonText = approveButtonText;
	}

	/**
	 * <br>this method defines the file filer needed by the dialog...
	 * @param filter the file filter
	 */
	public void setFileFilter(FileFilter filter){
		this.filter = filter;
	}

	/**
	 * <br>this method defines the overwrite text needed if instance choose a JFileChooser
	 * <br>for display the "save as" dialog
	 * @param overwriteText the overwrite text you want to display if user try to overwrite a file
	 */
	public void setOverwriteText(String overwriteText) {
		this.overwriteText = overwriteText;
	}
	
	public void setLocale(Locale locale) {
		this.locale = locale;
	}

	/**
	 * <br>this method returns the overwrite text needed by JFileChooser with the "save as" dialog"
	 * @return the overwrite text if it set, "!" if not
	 */
	private String getOverwriteText() {
		String result = null;
		if(overwriteText == null){
			result = "!";
		} else {
			result = overwriteText;
		}
		return result;
	}
	
	/**
	 * <br>this method convert the filter if it's set to a FileNameFilter instance
	 * @return a FilenameFilter instance according to filter
	 */
	private FilenameFilter fileFilterToFilenameFilter(){
		FilenameFilter returnFilter = null;
		if(filter != null){
			returnFilter = new FilenameFilter(){

				public boolean accept(File file, String name) {					
					return filter.accept(new File(name));
				}	
				
			};				
		}
		return returnFilter;
	}
	

	/**
	 * Method saveAsDialog
	 * <br>this method display a dialog for save as a file.
	 * <br>it displays a FileDialog if require condition is respected, a JfileChooser if not.
	 * <br>A file is return if user accepts to save its ile, null if not.
	 * @param parent 		the parent Frame
	 * @param title			the title of the dialog
	 * @param openedFile 	the file which want to save
	 * @return the saved file (created or existing file)
	 */
	public File showSaveAsDialog(Frame parent, String title, File openedFile){
		// the return file:
		File file = null;

		if(isMacCondition && lookAndFeelCondition){		
			FileDialog fileDialog = new FileDialog(parent, title, FileDialog.SAVE);
			if(!isWindowsCondition && filter != null) {
				// doesn't work on Microsoft Windows:
				fileDialog.setFilenameFilter(fileFilterToFilenameFilter());
			}
			
			if(locale != null) {
				fileDialog.setLocale(locale);
			}
			
			fileDialog.setLocationRelativeTo(parent);
			// set up the default file to save in...
			fileDialog.setFile(openedFile.getName());
			// we display the file dialog:
			fileDialog.setVisible(true);			
			// if user cancel the action, file return is null,
			if(fileDialog.getFile() != null){
				// we get back the file selected by user
				//if file already exists FileDialog manage it itself.
				file = new File(fileDialog.getDirectory() + fileDialog.getFile());
			}
		} else {		

			// we create a file chooser:
			JFileChooser fileChooser = null;
			if (openedFile != null) {
				fileChooser = new JFileChooser(openedFile.getParent());
				fileChooser.setSelectedFile(openedFile);
			} else {
				fileChooser = new JFileChooser();
			}
			// we personnalize it:
			fileChooser.setDialogTitle(title);
			if(approveButtonText != null) {
				fileChooser.setApproveButtonText(approveButtonText);
			}

			if(filter != null) {
				fileChooser.setFileFilter(filter);
			}

			// and display the dialog:
			int returnValue = fileChooser.showSaveDialog(parent);
			// if the user click OK:
			if(returnValue == JFileChooser.APPROVE_OPTION){
				file = fileChooser.getSelectedFile();
				if(file.exists()){						
					int result = JOptionPane.showConfirmDialog(fileChooser, file.getName() + " " + getOverwriteText(), "" , JOptionPane.YES_NO_OPTION);
					// if user don't accept to overwrite the file:
					if(result != JOptionPane.OK_OPTION){
						// set save to false:
						file = null;
					}
				}
			}				
		}
		return file;
	}

	public File showOpenDialog(Frame parent, String title, File openedFile){
		File file = null;
		if(isMacCondition && lookAndFeelCondition){
			FileDialog fileDialog = new FileDialog(parent, title, FileDialog.LOAD);
			if(!isWindowsCondition && filter != null) {
				// doesn't work on Microsoft Windows:
				fileDialog.setFilenameFilter(fileFilterToFilenameFilter());
			}
			
			if(locale != null) {
				fileDialog.setLocale(locale);
			}
			
			if(openedFile != null){	
				// we set the default opened directory:
				fileDialog.setDirectory(openedFile.getParent());
				// we set the default selected file:
				fileDialog.setFile(openedFile.getName());
			}
			
			fileDialog.setVisible(true);
			// fileDialog.setFile(null);
			// if user cancel the action, file is null,
			if(fileDialog.getFile() != null){
				// we get back the file selected by user
				file = new File(fileDialog.getDirectory() + fileDialog.getFile());
			}		
		} else {
			// create the file chooser:
			JFileChooser fileChooser = null;
			if (openedFile != null) {
				fileChooser = new JFileChooser(openedFile.getPath());
			} else {
				fileChooser = new JFileChooser();
			}
			// we set a filter on extension file:
			if (filter != null){
				fileChooser.setFileFilter(filter);
			}
			
			// we display the browser:
			int result = fileChooser.showOpenDialog(parent);
			if ( result == JFileChooser.APPROVE_OPTION) {
				// set result if user approves selection:
				file = fileChooser.getSelectedFile();
			}
		}	

		return file;
	}
}