/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
package sears.tools;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Properties;

/**
 * Class SearsProperties.
 * <br><b>Summary:</b><br>
 * This class contains the Sears Properties.
 */
public class SearsProperties {
    /**The properties*/
    private Properties properties;
    /**The instance of the class.*/
    private static SearsProperties instance = new SearsProperties();
    
    /**The config file name.*/
    private static final String CONFIG_FILE = "config.cfg";
    
    /**The config folder name.*/
    private static final String CONFIG_FOLDER = ".sears";
    
    
    /**The width suffix property.*/
    public static final String SUFFIX_WIDTH = "_width";
    /**The heigth suffix property.*/
    public static final String SUFFIX_HEIGTH = "_heigth";
    /**The posX suffix property.*/
    public static final String SUFFIX_POSX = "_posX";
    /**The posY suffix property.*/
    public static final String SUFFIX_POSY = "_posY";
    /**The last folder property*/
    public static final String LAST_FOLDER = "File_lastFolder";
    /**The language property*/
    public static final String LANGUAGE = "Sears_lang";
    /**The country property*/
    public static final String COUNTRY = "Sears_count";  
    /**The player property*/
    public static final String PLAYER_FULL_PATH = "Player_fullPath";
    /**The look and feel property.*/
    public static final String LOOK_AND_FEEL = "Sears_lookAndFeel";
    /**The DOS line separator property.*/
    public static final String DOS_LINE_SEPARATOR = "Sears_useDOSLineSeparator";
    /**The player selected.*/
    public static final String PLAYER_SELECTED = "Sears_selectedPlayer";  
    /**The update address.*/
    public static String UPDATE_ADDRESS = "Sears_update";    
    /**The key to the other player choice.*/
    public final static int KEY_OTHER = 1;
    /**The key to the VLC player choice.*/
    public final static int KEY_VLC = 0;
    /**The ICON_SET_FILE: The file that contains the icons.*/
    public static final String ICON_SET = "Sears_iconSet";
    /**The DEFAULT_ICON_SET: The default icon set.*/
    public static final String DEFAULT_ICON_SET = "Flat.jar";
    /** (<b>String</b>) RECENT_FILES: The RECENT_FILES */
    public static final String RECENT_FILES = "Sears_recentFiles";
    /** (<b>String</b>) SEARCH_WIDTH: the search area width */
    public static final String TOOLBAR_DIVIDER_LOCATION = "toolbar_divider_location";
    
    
    /**The VLC_RESTART: to indicate wether vlc should restart after a subtitle change.*/
    public static String VLC_RESTART = "Sears_vlcRestart";
    /**The VLC_PORT: the port to connect vlc.*/
    public static String VLC_PORT = "Sears_vlcPort";
   
    /**
     * Constructor SearsProperties.
     * <br><b>Summary:</b><br>
     * Constructor of the class.
     */
    public SearsProperties(){
        properties = loadProperties();
    }
    
    
    /**
     * Method loadProperties.
     * <br><b>Summary:</b><br>
     * Use this method to load the software properties.
     * @return  <b>Properties</b>   The loaded properties.
     */
    private Properties loadProperties() {
        //the result of the method.
        Properties result = new Properties();
        try {
        	//Load the property file.
            result.load(new FileInputStream(getConfigFile()));
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
        //return the result
        return result;
    }
    
    /**
     * Method saveProperties.
     * <br><b>Summary:</b><br>
     * This method saves the properties of Sears in the property file.
     */
    public static void saveProperties() {
        try {
            //save the properties.
            instance.properties.store(new FileOutputStream(getConfigFile()), "");
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    /**
     * Method getProperty.
     * <br><b>Summary:</b><br>
     * Return the sears property that correspond to the key. or "" value if not found.
     * @param key               The key to search.
     * @return <b>String</b>    The sears property that correspond to the key. or "" value if not found.
     */
    public static String getProperty(String key){
        return getProperty(key, "");
    }
    
    /**
     * Method getProperty.
     * <br><b>Summary:</b><br>
     * Return the sears property that correspond to the key. or default value if not found.
     * @param key               The key to search.
     * @param defaultValue      The default value to return if not found.
     * @return <b>String</b>    The sears property that correspond to the key. or default value if not found.
     */
    public static String getProperty(String key, String defaultValue){
        //The result of the method.
        String result = "";
        //try to get the result from the properties.
        result = instance.properties.getProperty(key, defaultValue);
        //return the result.
        return result;
    }
    
    /**
     * Method setProperty.
     * <br><b>Summary:</b><br>
     * Set the Sears property.
     * @param key       The key to set.
     * @param value     The value to associate.
     */
    public static void setProperty(String key, String value){
        instance.properties.setProperty(key, value);
    }
    
    public static void resetProperty() throws IOException {
    	getConfigFile().delete();
    	instance = new SearsProperties();
    }
    
    /**
     * Method getConfigFile.
     * <br><b>Summary:</b><br>
     * This method return the config file to be used with Sears.
     * It creates it if not found.
     * It uses the user.home property of the system.
     * @return  (<b>File</b>)   The config file to be used to load and save Sears propertiess.
     * @throws IOException
     */
    private static File getConfigFile() throws IOException{
    	//The result of the method.
    	File result = new File(System.getProperty("user.home")+File.separator+CONFIG_FOLDER+File.separator+CONFIG_FILE);
    	//check parent folder existence
        if(!result.getParentFile().exists()){
        	result.getParentFile().mkdirs();
        }
        //Then create the property file if does not exist.
        if(!result.exists()){
        	result.createNewFile();
        }
        //return the result.
        return result;
    }
}
