/////////////////////////////////////////////////
// This file is part of Sears project.
// Subtitle Editor And Re-Synch
// A tool to easily modify and resynch movies subtitles.
/////////////////////////////////////////////////
//This program is free software; 
//you can redistribute it and/or modify it under the terms 
//of the GNU General Public License 
//as published by the Free Software Foundation; 
//either version 2 of the License, or (at your option) any later version.
/////////////////////////////////////////////////
//Sears project is available under sourceforge
// at adress: http://sourceforge.net/projects/sears/
//Copyright (C) 2005 Booba Skaya
//Mail: booba.skaya@gmail.com
////////////////////////////////////////////////
package sears.tools;

import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import sun.net.www.content.text.PlainTextInputStream;

/**
 * Class Updater.
 * <br><b>Summary:</b><br>
 * This class permit to know if Sears has an available update.
 */
public class Updater {
    /**The update address.*/
    private String address;

    /**The JFrame parent, used to display message dialog.*/
    private JFrame parent;
    
    /**The default update address*/
    public static final String DEFAULT_UPDATE_ADDRESS = "http://nico.rouviere.free.fr/Sears/version";

    /**
     * Constructor Updater.
     * <br><b>Summary:</b><br>
     * Constructor of the class.
     * This will launch a thread that try to read the given address and retrieve the version.
     * If given version is lower that the one found at the given adress, it will show a message dialog.
     *
     * @param address           The address to look for the newest version number.
     * @param currentVersion    The current version.
     */
    public Updater(JFrame parent, String address, double currentVersion) {
        this.address = address;
        this.parent = parent;
        final double myVersion = currentVersion;
        //construct a thread that will reach the version without bloking the GUI.
        Thread updaterThread = new Thread("UpdaterThread") {
            public void run() {
                double version = getAddressVersion();
                if (myVersion < version) {
                    showUpdateAvailableMessage(version);
                }
            }
        };
        updaterThread.start();
    }

    /**
     * Method showUpdateAvailableMessage.
     * <br><b>Summary:</b><br>
     * This method displays a message to the user about the new available version.
     * @param version   The new version available.
     */
    protected void showUpdateAvailableMessage(double version) {
        JOptionPane.showConfirmDialog(parent
                                    , SearsResourceBundle.getResource("various_updateAvailable1")+" "+ version
                                    + " "+SearsResourceBundle.getResource("various_updateAvailable2")+"\n"
                                    + SearsResourceBundle.getResource("various_updateAvailable3")
                                    ,SearsResourceBundle.getResource("various_updateAvailableTitle")
                                    ,JOptionPane.DEFAULT_OPTION);
    }

    /**
     * Method getAddressVersion.
     * <br><b>Summary:</b><br>
     * This method retrieve the version at the updater address.
     * @return  <b>double</b>      The found version, or -1 if an error occurs.
     */
    protected double getAddressVersion() {
        //The result of the method.
        double result = -1;
        //Try to read a double in the found text.
        try {
            //retrieve the text at the given index.
            String versionString = getAddressText();
            result = Double.parseDouble(versionString);
        } catch (NumberFormatException e) {
            //we failed to get the version number return -1
            Trace.trace("The text found is not a double at address "+address, Trace.ERROR_PRIORITY);
        }
        //return the result
        return result;
    }

    /**
     * Method getAddressText.
     * <br><b>Summary:</b><br>
     * This method get the text at a given internet address.
     * or "" if an error occurs.
     * @return  <b>String</b>   The text found at the address.
     */
    public String getAddressText() {
        //The result of the method
        String result = "";
        try {
            //construc the URL to retrieve the text..
            URL url = new URL(address);
            //and retrieve the file at the given address.
            InputStreamReader reader = new InputStreamReader(((PlainTextInputStream) url.getContent()));
            int c = reader.read();
            while (c != -1) {
                result += (char) c;
                c = reader.read();
            }
        } catch (MalformedURLException e) {
            //The URL is not correct. do nothing.
            Trace.trace("The given update address is not valid :"+address+" : "+e.getMessage(), Trace.ERROR_PRIORITY);
        } catch (IOException e) {
            //An error occurs during reading, do nothing.
            Trace.trace("An exception occurs during update reading: "+e.getMessage() , Trace.ERROR_PRIORITY);
        }
        //return the result.
        return result;
    }
}
